C
C                  M   M   CCC  M   M   AA    GGG
C                  MM MM  C     MM MM  A  A  G  
C                  M M M  C     M M M  AAAA  G GGG
C                  M   M  C     M   M  A  A  G   G
C  *************** M   M   CCC  M   M  A  A   GGG ****************
C *                                                               *
C *   MCMAG - A FORTRAN PROGRAM TO SIMULATE MAGNETIC STRUCTURES   *
C *                   VAX-FORTRAN VERSION 8801                    *
C *                                                               *
C *                    written by P. Lacorre                      *
C *  Institut Laue-Langevin, 156X, 38042 Grenoble Cedex - FRANCE  *
C *                             and                               *
C *  Laboratoire des Fluorures UA CNRS 449, Faculte des Sciences  *
C *         Route de Laval, 72017 Le Mans Cedex - FRANCE          *
C *                                                               *
C *****************************************************************
C                             
C      MCMAG is a Fortran program to simulate magnetic models from 
C  postulated coupling constants. Ground state configurations are
C  obtained by a simulated annealing algorithm [S. Kirkpatrick,
C  C.D. Gellatt Jr. and M.P. Vecchi, Science, 220 (1983) 671] and
C  the configuration space is explored with random walks by using
C  Metropolis importance sampling [N. Metropolis, A.W. Rosenbluth,
C  M.N. Rosenbluth, A.H. Teller and E. Teller, J. Chem. Phys. 21
C  (1953) 1087]. Any kind of 1D, 2D or 3D periodic model, magnetic 
C  cluster as well as real magnetic structure can be simulated 
C  without limitation of interacting distance.
C
C  Further details on the method used for this purpose are given
C  in the paper :
C  MCMAG: a computer program to simulate magnetic structures.
C  P. Lacorre and J. Pannetier, J. Magn. Magn. Mat. 71 (1987) 63.
C
C  Topological and magnetic parameters must be stored in a file
C  MYFILE.DAT (where MYFILE is a 6 characters filename given by
C  user). This input file contains a list of the magnetic
C  sites of the structure, together with a list of their neighbours 
C  and corresponding exchange integrals, as well as anisotropy
C  coefficients and spin amplitudes (for a complete description see 
C  the manual MCMAG.MAN). No symmetry element is taken into account
C  by the program: all spins are independent.
C  The so-called basic unit cell (b.u.c.) which is described in
C  the input file represents the smallest sample that can be used
C  for a simulation. Larger size samples are built up from the
C  stacking of such unit cells along the three crystallographic
C  axes and defined by the number of cells along these axes (namely
C  LU, LV and LW for axes U, V and W respectively). Three kinds of 
C  boundary conditions are allowed:
C       - free edges
C       - periodic boundary conditions
C       - mixed conditions (<=> free edges along W and periodic
C         boundary conditions along U and V)
C		
C  The upper limit in sample size is imposed by the array dimension
C  of the program. In order to allow an easy modification of these
C  limits (which could be convenient for the study of specific cases)
C  they have been introduced as parameters in the aside file 
C  MCMAG.INC, which is included to the source file during the compi-
C  lation.These upper limits are called :
C       - NUMAX : maximal number of cells along the U axis
C       - NVMAX : maximal number of cells along the V axis
C       - NWMAX : maximal number of cells along the W axis
C       - NAMAX : maximal number of sites per cell
C       - NNMAX : maximal number of neighbours per site
C
C  The file MCMAG.INC contains the following parameters:
C     PARAMETER(
C    1          NUMAX=5,      ! maximal number of cells along the U axis
C    1          NVMAX=5,      ! maximal number of cells along the U axis
C    1          NWMAX=5,      ! maximal number of cells along the U axis
C    1          NAMAX=64,     ! maximal number of sites per cell
C    1          NNMAX=50,     ! maximal number of neighbours per site
C    1          NMOMAX=200,   ! maximal number of isotropic coupling 
C    1                        !   constants which can be modified 
C    1                        !   interactively
C    1          NAL=8,        ! maximal number of site moments to be
C    1                        !   written on a single line of the output 
C    1                        !   file MYFILE.LIS
C    1          I01=31,       ! input unit number for file MYFILE.DAT
C    1          I02=32,       ! input unit number for file MYFILE.INI
C    1          I03=33,       ! output unit number for file MYFILE.RES
C    1          I04=34,       ! output unit number for file MYFILE.SPI
C    1          IBA=35,       ! output unit number for file MYFILE.REC
C    1          LPT=7,        ! output unit number for file MYFILE.LIS
C    1          ITTI=5,       ! input unit number for terminal
C    1                        !   (interactive mode)
C    1          ITTO=6        ! output unit number for terminal
C    1                        !   (interactive mode)
C    1                   )
C
C  The initial configuration of spins is either randomly generated
C  by the program or can be imposed (file MYFILE.INI).
C  During the simulation, new orientations of spins are drawn at
C  random and adopted or rejected (according to Boltzmann statistics)
C  in order to progressively minimize the energy of the system.
C  The Hamiltonian used for the energy calculation includes three
C  terms :
C       - a general two-spin coupling term with facility for anisotropic
C         asymmetric exchange
C       - a single-ion anisotropy term
C       - an applied magnetic field term
C  The energy formula may be expressed as follows:
C  (with the convention that a vector V' represents the transposed 
C  form of a vector V)
C
C       E = - 0.5*Si'[Jij]Sj + ((Di'Si)**2)/|Di| - H'Si
C
C       where :
C          Si,Sj  are vectors representing two interacting spins
C          [Jij]  is the coupling tensor:
C                                                         Jxx Jxy Jxz
C                 for every <i,j> pair, [Jij] represents  Jyx Jyy Jyz
C                                                         Jzx Jzy Jzz
C          Di     is the vector of single-ion anisotropy
C          H      is the vector of applied magnetic field
C
C  The reference used for spins is totally independent of the crys-
C  tallographic one: spins are always generated and expressed in an 
C  orthonormal coordinate system during the simulation process. The 
C  same orthonormal coordinate system is used for the anisotropy 
C  coefficients and for the applied magnetic field if any. For sake 
C  of convenience, a possibility is offered at the end of the
C  simulation, to project the magnetic moments on the real crystal 
C  axes and to rotate the whole system to align one moment along a
C  desired direction of the crystal frame of reference. This option
C  facilitates the comparison between the result of simulation and
C  real or theoretical arrangements of magnetic moments.
C
C  The external conditions of the simulation (cooling rate, varying 
C  field ...) are introduced in an interactive way, as sequences of
C  iteration on temperature at H=cst or on magnetic field at T=cst.
C  The arrangements of magnetic moments and last spin configuration
C  are stored during the process and at the end of simulation, respec-
C  tively. Averaged characteristics such as magnetic susceptibility 
C  or specific heat and also energy and magnetization are collected 
C  at every step of the iterations.
C
C  The following files are generated during or at the end of the
C  simulation :
C
C  - MYFILE.REC : contains the sequence of instructions given by
C                 user during the run of MCMAG (can be used as ins-
C                 truction file for further batch runs)
C
C  - MYFILE.LIS : contains all informations about the arrangement of
C                 magnetic moments and the averaged characteristics
C                 collected at every step of the simulation process
C
C  - MYFILE.RES : created at user's request, this file stores averaged
C                 characteristics collected at every temperature or
C                 field step, for further analysis or plotting
C
C  - MYFILE.SPI : created at the end of simulation, it contains the
C                 last generated spin configuration of the simulation. 
C                 The file format is the same as that of file MYFILE.INI,
C                 which allows to split up a simulation in several runs
C
C       The main program calls the following subroutines :
C
C  READ_FILE   : read input data and simulation parameters
C  COMP_TEST   : check internal consistency of data
C  MOD_INT     : if activated, modify some selected isotropic coupling
C  GEN_SPI     : generate or read initial spin configuration
C  MC_CYCLE    : execute the Monte Carlo loops
C  EN_TOT      : compute the total energy of the system
C  CONST_FUNCT : compute the constraint function (isotropic coupling)
C  OUT_RES     : output averaged characteristics
C  OUT_CONF    : output the configuration of magnetic moments
C  PRO_ROTA    : if selected, project moments on crystal cell (rotate)
C
C  These subroutines call other subroutines or function :
C
C  BOUND_COND : compute the selected boundary conditions
C  INV_MAT    : invert matrix
C  ENn_CALC   : compute energy difference between new and old spin conf.
C  RANnD      : generate a pseudo-random vector
C  RANF       : generate a pseudo-random number between 0 and 1
C
C  Four subroutines ENn_CALC and four subroutines RANnD are included in
C  the program (n = 1, 2, 3 or q). Only one among the four is activated
C  during a simulation, depending on the selected spin model, namely :
C  - Ising spin model           (n = 1)
C  - XY spin model              (n = 2)
C  - Heisenberg spin model      (n = 3)
C  - q-sates planar Potts model (n = q)
C
C***********************************************************************
C
C  The following lines give a list of the main parameters and arrays
C  used in MCMAG (main program and subroutines) :
C
C    NAV                   : indices of the neighbours of a site
C
C    PT	                   : pointer of NAV
C
C    NU,NV,NW              : absolute indices of neighbouring cells
C
C    NAT                   : indices of sites in a cell
C
C    NVS                   : total number of neighbours of a site
C
C    M                     : relative indices of neighbouring cells
C
C    AA,AD                 : matrices for inversion (to rotate moments)
C
C    TCO                   : test of internal compatibility
C                            (topological self-consistency)
C
C    TCOJX,JY,JZ,JXY,JXZ   : test of internal compatibility (interactions)
C    ...JYX,JYZ,JZX,JZY
C
C    AMPS,AMPS2	           : spin amplitude
C
C    SX,SY,SZ              : spin components
C
C    SOMX,SOMY,SOMZ        : components of magnetic moment
C    COMX                  : storage of SOMX when SOMX is used for
C                            another purpose
C
C    TMOM                  : total magnetic moment
C
C    SOMRX,SOMRY,SOMRZ     : components of magnetic moment after rotation
C
C    SOMX2,SOMY2,SOMZ2     : variance and standard deviation of magnetic
C			     moment
C
C    SOMRX2,SOMRY2,SOMRZ2  : variance and standard deviation of magnetic
C			     moment after rotation
C
C    ISIGX,ISIGY,ISIGZ     : 1000*(standard deviation of components)
C    ISIGM                 : 1000*(standard deviation of total moment)
C
C    AJSX,AJSY,AJSZ        : components of the local molecular field
C
C    AJXX,AJXY,AJXZ        : elements of the exchange tensor
C    AJYX,AJYY,AJYZ		
C    AJZX,AJZY,AJZZ		
C
C    AJJX,AJJY,AJJZ        : diagonal elements of the exchange tensor
C                            or storage of isotropic (AJXX,AJYY,AJZZ)
C                            before eventual modifications
C
C    D              	   : anisotropy coefficient
C
C    DDIX,DDIY,DDIZ        : direction of anisotropy
C    D0	                   : normalizing factor for the direction of 
C                            anisotropy
C
C    DX,DY,DZ              : normalized direction of anisotropy
C
C
C***********************************************************************











C                  *******************************
C                  *                             *
C                  *        MAIN PROGRAM         *
C                  *                             *
C                  *******************************






      INCLUDE 'MCMAG.INC/LIST'

      DIMENSION
C    &   JOUR(3),HEURE(2),
C    &   NAV(NAMAX,NNMAX),
C    &   NU(NNMAX),NV(NNMAX),NW(NNMAX),
C    &   PT(NNMAX),
C    &   NAT(NAMAX),
C    &   NVS(NAMAX),
C    &   M(NAMAX,NNMAX,3),
C    &   AD(3,3),
C    &   TCO(3),
C    &   AMPS(NAMAX),AMPS2(NAMAX),
C    &   AJSX(NAMAX),AJSY(NAMAX),AJSZ(NAMAX),
C    &   AJJX(NAMAX,NNMAX),
C    &   AJJY(NAMAX,NNMAX),
C    &   AJJZ(NAMAX,NNMAX),
C    &   AJXX(NAMAX,NNMAX),
C    &   AJYY(NAMAX,NNMAX),
C    &   AJZZ(NAMAX,NNMAX),
C    &   AJXY(NAMAX,NNMAX),
C    &   AJXZ(NAMAX,NNMAX),
C    &   AJYX(NAMAX,NNMAX),
C    &   AJYZ(NAMAX,NNMAX),
C    &   AJZX(NAMAX,NNMAX),
C    &   AJZY(NAMAX,NNMAX),
C    &   SOMX(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   COMX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
C    &   SOMY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
C    &   SOMZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
C    &   TMOM(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
C    &   SOMRX(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   SOMRY(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   SOMRZ(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   SOMX2(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   SOMY2(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   SOMZ2(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   SOMRX2(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   SOMRY2(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   SOMRZ2(NAMAX,NUMAX,NVMAX,NWMAX),
C    &   ISIGX(NAMAX),
C    &   ISIGY(NAMAX),
C    &   ISIGZ(NAMAX),
C    &   ISIGM(NAMAX),
C    &   IUM(NMOMAX),IVM(NMOMAX),IWM(NMOMAX),
C    &   NM(NMOMAX),
C    &   NUMB(NMOMAX),
C    &   AJMOD(NMOMAX),
C    &   FJSX(NAMAX),FJSY(NAMAX),FJSZ(NAMAX)
     &   D(NAMAX),
     &   DDIX(NAMAX),DDIY(NAMAX),DDIZ(NAMAX),
     &   DX(NAMAX),DY(NAMAX),DZ(NAMAX),
     &   D0(NAMAX),
     &   SX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1)



      INTEGER U,V,W,LU,LV,LW,IT,NOM

C     INTEGER*2 I1,I2
C
C     REAL*8 ENM,ENM2,EN2M,MAG2,MAGI2,TCO,TCOJX,TCOJY,TCOJZ,
C    &       TCOJXY,TCOJXZ,TCOJYZ,TCOJYX,TCOJZX,TCOJZY,
C    &       AA(3,3),AC(3,3),WK(3),AB(3,3),EN,ENMIN
C
C
      REAL*4 T,COEF,TFIN,HK,HX0,HY0,HZ0,HSTA,HSTE,HFIN
      REAL*8 PI
      CHARACTER TITL1*80,TITL2*80,TITL*50,ITIT1*80,ITIT2*80,ITIT3*80,
     &          TITL3*80,LFIL*10,BFIL*10,NFIL*6,INI*10,RESU*10,SPI*10,
     &          SPIMOD*12,AVCHAR*9,BCOND*10,CODT*1


      COMMON 
C    &       /COM4/M
C    &       /COM8/NVS
C    &       /CONDL/NU,NV,NW
C    &       /TEST/NAV,AJJX,AJJY,AJJZ,AJXY,AJXZ,AJYZ,
C    &       AJYX,AJZX,AJZY,PT,NAT,TCO,TCOJX,TCOJY,TCOJZ,
C    &       TCOJXY,TCOJXZ,TCOJYZ,TCOJYX,TCOJZX,TCOJZY
C    &       /CYC1/AJXX,AJYY,AJZZ
C    &       /CYC8/DELTA,X,Y,Z,NFIP,NMULT
C    &       /CYC2/AJMOD,IUM,IVM,IWM,NM,NUMB,IM
C    &       /CYC9/SOMX,AJSX,AJSY,AJSZ
C    &       /CYC7/ENM,EN2M,MAG2
C    &       /COM5/AMPS,AMPS2
C    &       /CYC4/SOMY,SOMZ
C    &       /CYC5/SOMX2,SOMY2,SOMZ2
C    &       /CYC6/EN
C    &       /SOMM1/TMOM,AMX,AMY,AMZ
C    &       /SOMM2/COMX
C    &       /PRO/SOMRX,SOMRY,SOMRZ,SOMRX2,SOMRY2,SOMRZ2
C    &       /FCO/FCONST,RC
     &       /COND2/IANB
     &       /COM2/NA,IANI
     &       /PALL/LU,LV,LW,IRAN
     &       /TEST2/JCOD
     &       /READ1/NMOT,NMA
     &       /READ4/D,DX,DY,DZ
     &       /READ5/DDIX,DDIY,DDIZ,D0
     &       /READ2/TITL1,TITL2,NFIL,BFIL
     &       /READ3/INI,IANS
     &       /CYC10/IT,NOM
     &       /COM6/NPAJ
     &       /CYC11/NFA,HX0,HY0,HZ0
     &       /CYC3/SX,SY,SZ
     &       /COM7/NTA
     &       /COM1/T
     &       /FIELD/HX,HY,HZ
     &       /MOD1/ANSMOD
     &       /RES2/SNOM
     &       /RES3/HK,HDIX,HDIY,HDIZ,J1,J2,J3
     &       /RES1/LAST,ANSRF,ANSNEV
     &       /RAND/PI,NQ
     &       /WRIT/IWMX,IWMXY



      WRITE(ITTO,1233)               ! INTRODUCTORY TITLE
      WRITE(ITTO,3000)
      WRITE(ITTO,695)
 1235 WRITE(ITTO,1)                  ! INPUT FILE NAME ?
      READ(ITTI,2)NFIL
      OPEN(UNIT=I01,STATUS='OLD',FILE=NFIL,READONLY,ERR=1235)
      READ(I01,11)TITL1              ! READ FIRST TITLE OF THE INPUT FILE
      READ(I01,11)TITL2              ! READ SECOND TITLE OF THE INPUT FILE
      WRITE(ITTO,595)
 1101 WRITE(ITTO,2004)TITL1
      WRITE(ITTO,2004)TITL2 
      WRITE(ITTO,595)
      WRITE(ITTO,31)                 ! IS IT THE RIGHT FILE ?
      READ(ITTI,147)ANSI
      IF ((ANSI.NE.'Y').AND.(ANSI.NE.'y').AND.
     &   (ANSI.NE.'N').AND.(ANSI.NE.'n')) GO TO 1101
      IF ((ANSI.EQ.'N').OR.(ANSI.EQ.'n')) THEN
        CLOSE (UNIT=I01)
        GO TO 1235
      END IF
      LFIL = NFIL//'.LIS'
      OPEN(UNIT=LPT,STATUS='NEW',FILE=LFIL)

      CALL READ_FILE                 ! READ THE DATA FILE AND
                                     ! PARAMETERS OF THE SIMULATION

      CALL COMP_TEST                 ! TEST OF INTERNAL COMPATIBILITY
                                     ! CONCERNING TOPOLOGY AND INTERACTIONS

 1102 WRITE(ITTO,4001)               ! ISING, XY, HEISENBERG OR q-STATE
	                             ! POTTS PLANAR SPINS ?
      READ(ITTI,708)IRAN
      IF ((IRAN.LT.1).OR.(IRAN.GT.4)) GO TO 1102

      IF (IRAN.EQ.4) THEN            ! q-STATE POTTS PLANAR SPINS
        PI = 3.141592654
 1103   WRITE(ITTO,4011)
        READ(ITTI,*)NQ
        IF (NQ.LT.2) GO TO 1103
      END IF

      IF (IRAN.EQ.1) THEN            ! ISING SPINS
        SPIMOD='ISING'
        ICOU = 0
        DO I=1,NA
          IF (D0(I).EQ.0) THEN
            DX(I)=1.
            ICOU = ICOU+1
          END IF
        END DO
        IF (ICOU.EQ.NA) IWMX=1

      ELSE IF (IRAN.EQ.2) THEN       ! XY SPINS
        SPIMOD='XY PLANAR'
        IWMXY=1

      ELSE IF (IRAN.EQ.3) THEN       ! HEISENBERG SPINS
        SPIMOD='HEISENBERG'

      ELSE IF (IRAN.EQ.4) THEN       ! q-STATE POTTS PLANAR SPINS
        SPIMOD='-STATE POTTS'
        IWMXY=1

      END IF

      BFIL=NFIL//'.REC'
      OPEN(UNIT=IBA,STATUS='NEW',FILE=BFIL)
      WRITE(IBA,4003)NFIL,IRAN

      IF (IRAN.EQ.4) THEN
        WRITE(IBA,*)NQ
      END IF
	
      WRITE(ITTO,2001)               ! TITLE OF SIMULATION ?
      READ(ITTI,2220)TITL
      WRITE(IBA,2220)TITL
 1104 WRITE(ITTO,14)NUMAX,NVMAX,NWMAX! SIZE OF THE SAMPLE ?
      READ(ITTI,*,ERR=1104)LU,LV,LW
      IF ((LU.GT.NUMAX).OR.(LV.GT.NVMAX).OR.(LW.GT.NWMAX)) GO TO 1104
      IF ((LU.LT.1).OR.(LV.LT.1).OR.(LW.LT.1)) THEN
        WRITE(ITTO,888)
        GO TO 1104
      END IF
      WRITE(IBA,*)LU,LV,LW
      NMA=LU*LV*LW
      NTA=NA*NMA
      ANSMOD='N'
      IF (JCOD.EQ.0) THEN
 1105   WRITE(ITTO,7700)             ! MODIFY SOME INTERACTIONS ?
        READ(ITTI,147)ANSMOD
        IF ((ANSMOD.NE.'Y').AND.(ANSMOD.NE.'y').AND.
     &     (ANSMOD.NE.'N').AND.(ANSMOD.NE.'n')) GO TO 1105
        WRITE(IBA,147)ANSMOD
        IF ((ANSMOD.EQ.'Y').OR.(ANSMOD.EQ.'y')) THEN

          CALL MOD_INT               ! POSSIBILITY TO MODIFY LOCALLY
                                     ! SOME INTERACTIONS (FOR ISOTROPIC
                                     ! EXCHANGE ONLY)

        END IF
      END IF
      INI=NFIL//'.INI'
 1106 WRITE(ITTO,707)INI             ! INITIAL SPIN CONFIGURATION ?
      READ(ITTI,147)IANS
      IF ((IANS.NE.'R').AND.(IANS.NE.'r').AND.
     &   (IANS.NE.'I').AND.(IANS.NE.'i')) GO TO 1106
      WRITE(IBA,147)IANS

      IF ((IANS.EQ.'I').OR.(IANS.EQ.'i')) THEN
        OPEN(UNIT=I02,STATUS='OLD',FILE=INI,READONLY,ERR=1106)
        READ(I02,2222)ITIT1,ITIT2,ITIT3
      END IF

      CALL GEN_SPI                   ! READ OR GENERATE THE INITIAL
                                     ! CONFIGURATION OF SPINS

 1107 WRITE(ITTO,4)                  ! BOUNDARY CONDITIONS ?
      READ(ITTI,889,ERR=1107)IANB
      WRITE(IBA,889)IANB

      IF (IANB.LT.0) THEN            ! MIXED BOUNDARY CONDITIONS
        BCOND = 'MIXED'

      ELSE IF (IANB.EQ.0) THEN       ! FREE EDGES
        BCOND = 'FREE EDGES'

      ELSE IF (IANB.GT.0) THEN       ! PERIODIC BOUNDARY CONDITIONS
        BCOND = 'PERIODIC'

      END IF

 1110 WRITE(ITTO,9261)               ! REFERENCE UNITS FOR OUTPUT ?
      READ(ITTI,708,ERR=1110)INFA  
      IF ((INFA.LT.1).OR.(INFA.GT.4)) GO TO 1110
      WRITE(IBA,708)INFA             
      IF (INFA.EQ.1) THEN            ! 1 = PER SAMPLE
        NFA=1         
        AVCHAR = 'SAMPLE'
      ELSE IF (INFA.EQ.2) THEN       ! 2 = PER UNIT CELL
        NFA=NMA       
        AVCHAR = 'UNIT CELL'
      ELSE IF (INFA.EQ.3) THEN       ! 3 = PER SITE
        NFA=NTA       
        AVCHAR = 'SITE'
      ELSE IF (INFA.EQ.4) THEN       ! 4 = PER MOLE
        NFA=NMA*NMOT  
        AVCHAR = 'MOLE'
      END IF

      CALL EN_TOT(1)                 ! CALCULATE THE TOTAL ENERGY
                                     ! WITH THE CURRENT MAGNETIC FIELD

C
C***** SUMMARIZE SIMULATION PARAMETERS *****
C
      WRITE(LPT,596)
      WRITE(LPT,2003)TITL
      WRITE(LPT,2015)LU,LV,LW,NA,NMOT,AVCHAR,BCOND
      IF (NQ.NE.0) THEN
        WRITE(LPT,2014)NQ,SPIMOD
      ELSE
        WRITE(LPT,2016)SPIMOD
      END IF
C
C*****
C

      WRITE(ITTO,1262)               ! INDICES OF THREE SITES ?
                                     ! (FOR OUTPUT OF RESULTING MOMENTS)
      READ(ITTI,*)J1,J2,J3
      WRITE(IBA,*)J1,J2,J3
      IF((J1.LE.0).OR.(J1.GT.NA))J1=1
      IF((J2.LE.0).OR.(J2.GT.NA))J2=2
      IF((J3.LE.0).OR.(J3.GT.NA))J3=3

 1108 WRITE(ITTO,41)                 ! COOLING RATE ?
C
C   KIRKPATRICK'S COOLING(HEATING) SCHEME : T(n+1) = COEF*T(n)
C
      READ(ITTI,*,ERR=1108)T,COEF,TFIN
      WRITE(IBA,*)T,COEF,TFIN
      ANSH = 'T'                     ! ANSH=T => ITERATION ON TEMPERATURE
      COD = 'M'                      ! COD=M => MULTIPLICATIVE COEFFICIENT
      CODT = 'x'

  706 WRITE(ITTO,112)                ! MAGNETIC FIELD ?
      READ(ITTI,*,ERR=706)HK,HX0,HY0,HZ0
      IF ( (HK.NE.0).AND.(HX0.EQ.0).AND.
     &    (HY0.EQ.0).AND.(HZ0.EQ.0) ) GO TO 706
      IF ( (HK.EQ.0).AND.(HX0.EQ.0).AND.
     &    (HY0.EQ.0).AND.(HZ0.EQ.0) ) HX0 = 1.
      WRITE(IBA,*)HK,HX0,HY0,HZ0

  518 WRITE(ITTO,501)                ! NUMBER OF MONTE-CARLO CYCLES ?
      READ(ITTI,*,ERR=518)IT,NOM
      WRITE(IBA,*)IT,NOM
      SNOM=FLOAT(IT-NOM)

      WRITE(LPT,2112)                ! PARAMETERS FOR CURRENT SCHEME
      WRITE(LPT,2013)IT,NOM,T,TFIN,CODT,COEF,HK,HK
      IF (HK.NE.0) THEN
        WRITE(LPT,2031)HX0,HY0,HZ0
      END IF
      WRITE(LPT,2030)

  243 HM0=SQRT(HX0*HX0+HY0*HY0+HZ0*HZ0)
      HDIX=HX0/HM0
      HDIY=HY0/HM0
      HDIZ=HZ0/HM0
      HI=HK/14.88732                 ! CONVERSION HK(Kelvin)->HI(kOe)
      HX=HDIX*HI                     ! HX,HY,HZ: EFFECTIVE COMPONENTS
      HY=HDIY*HI                     ! OF THE MAGNETIC FIELD
      HZ=HDIZ*HI
      WRITE(LPT,495)

 1109 WRITE(ITTO,568)                ! OUTPUT OF MAGNETIC MOMENTS ?
      READ(ITTI,147)ANSNEV
      IF ((ANSNEV.NE.'A').AND.(ANSNEV.NE.'a').AND.
     &    (ANSNEV.NE.'E').AND.(ANSNEV.NE.'e').AND.
     &    (ANSNEV.NE.'N').AND.(ANSNEV.NE.'n')) GO TO 1109
      WRITE(IBA,147)ANSNEV

      RESU = NFIL//'.RES'
 1111 WRITE(ITTO,261)RESU            ! AVERAGED CHARACTERISTICS IN A FILE ?
      READ(ITTI,147)ANSRF
      IF ((ANSRF.NE.'Y').AND.(ANSRF.NE.'y').AND.
     &    (ANSRF.NE.'N').AND.(ANSRF.NE.'n')) GO TO 1111
      WRITE(IBA,147)ANSRF

      IF ((ANSRF.NE.'Y').AND.(ANSRF.NE.'y')) GO TO 503

C
C***** OPENING AND FIRST LINES OF THE AVERAGED CHARARACTERISTICS FILE *****
C

      OPEN(UNIT=I03,STATUS='NEW',FILE=RESU)
      OU3=1
      TITL3 = SPIMOD//' SPINS - '//TITL
      WRITE(ITTO,1261)RESU
      NRES=NRES+1
      IF (NRES.EQ.1) THEN
        WRITE(I03,2012)NFIL,TITL1
        WRITE(I03,2008)TITL2
        WRITE(I03,2088)TITL
        IF (NQ.NE.0) THEN
          WRITE(I03,2009)NQ,SPIMOD,LU,LV,LW,AVCHAR
        ELSE
          WRITE(I03,2099)SPIMOD,LU,LV,LW,AVCHAR
        END IF
      END IF
      WRITE(I03,368)NRES,IT,NOM,HX0,HY0,HZ0
      WRITE(I03,264)J1,J2,J3

C
C***** AVERAGED CHARACTERISTICS FILE OPENED *****
C

  503 IF (HK.NE.0) THEN
        WRITE(LPT,484)T,HK,HI,HX0,HY0,HZ0
        WRITE(ITTO,484)T,HK,HI,HX0,HY0,HZ0
      ELSE
        WRITE(LPT,494)T,HK,HI
        WRITE(ITTO,494)T,HK,HI
      END IF

      WRITE(ITTO,1494)

      CALL EN_TOT(1)                 ! CALCULATE THE TOTAL ENERGY
                                     ! WITH THE CURRENT MAGNETIC FIELD

      CALL MC_CYCLE                  ! EXECUTE THE MONTE-CARLO LOOPS

C
C***** TEST IF THE CURRENT ITERATION IS THE LAST ONE ***** 
C

      IF ((ANSH.EQ.'F').OR.(ANSH.EQ.'f')) THEN
        HKI=HK+HSTE
        IF ((HSTE.GT.0).AND.(HKI.GT.HFIN)) LAST=1
        IF ((HSTE.LT.0).AND.(HKI.LT.HFIN)) LAST=1

      ELSE
        IF ((COD.EQ.'M').OR.(COD.EQ.'m')) THEN
          TCI=T*COEF
          IF ((COEF.GT.1).AND.(TCI.GT.TFIN)) LAST=1
          IF ((COEF.LT.1).AND.(TCI.LT.TFIN)) LAST=1
        ELSE
          TCI=T+COEF
          IF ((COEF.GT.0).AND.(TCI.GT.TFIN)) LAST=1
          IF ((COEF.LT.0).AND.(TCI.LT.TFIN)) LAST=1
        END IF

      END IF

C
C***** END OF LAST ITERATION TEST *****
C

      CALL OUT_CONF                  ! OUTPUT CONFIGURATION OF MOMENTS

      IF (JCOD.EQ.0) THEN

        CALL CONST_FUNCT             ! CALCULATE THE CONSTRAINT FUNCTION
                                     ! OF THE SYSTEM (FOR ISOTROPIC
                                     ! EXCHANGE ONLY)

      END IF

      CALL OUT_RES                   ! OUTPUT AVERAGED CHARACTERISTICS

      CALL EN_TOT(2)                 ! CALCULATE THE TOTAL ENERGY
                                     ! WITH THE CURRENT MAGNETIC FIELD


C
C***** INCREMENT FIELD OR TEMPERATURE *****
C

      IF ((ANSH.EQ.'F').OR.(ANSH.EQ.'f')) THEN
                                     ! ANSH=F => ITERATION ON FIELD

        HK=HK+HSTE                   ! HK = NEW MAGNETIC FIELD
        HI=HK/14.88732
        HX=HDIX*HI
        HY=HDIY*HI
        HZ=HDIZ*HI

        IF ((HSTE.GT.0).AND.(HK.LE.HFIN)) GO TO 503
        IF ((HSTE.LT.0).AND.(HK.GE.HFIN)) GO TO 503

      ELSE                           ! ANSH=T => ITERATION ON TEMPERATURE
        IF ((COD.EQ.'M').OR.(COD.EQ.'m')) THEN
                                     ! COD=M => MULTIPLICATIVE COEFFICIENT
                                     ! KIRKPATRICK'S COOLING(HEATING) SCHEME
          TA=T*COEF
          IF ((COEF.GT.1).AND.(TA.LE.TFIN)) THEN
            T=TA
            GO TO 503
          END IF
          IF ((COEF.LT.1).AND.(TA.GE.TFIN)) THEN
            T=TA
            GO TO 503
          END IF
        ELSE                         ! COD=A => ADDITIVE COEFFICIENT
                                     ! LINEAR COOLING(HEATING) SCHEME
          TA=T+COEF
          IF ((COEF.GT.0).AND.(TA.LE.TFIN)) THEN
            T=TA
            GO TO 503
          END IF
          IF ((COEF.LT.0).AND.(TA.GE.TFIN)) THEN
            T=TA
            GO TO 503
          END IF
        END IF
      END IF

C
C***** END OF INCREMENTATION *****
C

      LAST=0
 1112 WRITE(ITTO,146)                ! DO YOU WANT TO CONTINUE ?
      READ(ITTI,147)ANSC
      WRITE(LPT,1551)
      IF ((ANSC.NE.'Y').AND.(ANSC.NE.'y').AND.
     &     (ANSC.NE.'N').AND.(ANSC.NE.'n')) GO TO 1112
      WRITE(IBA,147)ANSC

      IF ((ANSC.EQ.'Y').OR.(ANSC.EQ.'y')) THEN
 1113   WRITE(ITTO,148)              ! MODIFY T OR H ?
        READ(ITTI,147)ANSH
        IF ((ANSH.NE.'F').AND.(ANSH.NE.'f').AND.
     &     (ANSH.NE.'T').AND.(ANSH.NE.'t')) GO TO 1113
        WRITE(IBA,147)ANSH

        IF ((ANSH.EQ.'T').OR.(ANSH.EQ.'t')) THEN

 1114     WRITE(ITTO,506)            ! CODE FOR HEATING/COOLING ?
          READ(ITTI,147)COD
          IF ((COD.NE.'M').AND.(COD.NE.'m').AND.
     &       (COD.NE.'A').AND.(COD.NE.'a')) GO TO 1114
          WRITE(IBA,147)COD
 1115     WRITE(ITTO,606)            ! HEATING/COOLING FACTOR and FINAL TEMP. ?
C
C   KIRKPATRICK'S COOLING(HEATING) SCHEME : T(n+1) = COEF*T(n)
C   LINEAR COOLING(HEATING) SCHEME : T(n+1) = COEF+T(n)
C
          READ(ITTI,*,ERR=1115)COEF,TFIN
          WRITE(IBA,*)COEF,TFIN

          IF ((COD.EQ.'M').OR.(COD.EQ.'m')) THEN
            T=T*COEF                 ! FIRST TEMPERATURE OF THE NEW
                                     ! ITERATION
            CODT = 'x'

          ELSE	
            T=T+COEF                 ! FIRST TEMPERATURE OF THE NEW
                                     ! ITERATION
            CODT = '+'

          END IF

          GO TO 706

        END IF

 1116   WRITE(ITTO,242)              ! FIELD DIRECTION ?
        READ(ITTI,*,ERR=1116)HX0,HY0,HZ0
        IF ( (HX0.EQ.0).AND.(HY0.EQ.0).AND.(HZ0.EQ.0) ) GO TO 1116
        WRITE(IBA,*)HX0,HY0,HZ0
 1117   WRITE(ITTO,241)              ! FIELD INCREMENT ?
        READ(ITTI,*,ERR=1117)HSTA,HSTE,HFIN
        WRITE(IBA,*)HSTA,HSTE,HFIN
        HK=HSTA

  519   WRITE(ITTO,501)              ! NUMBER OF MONTE-CARLO CYCLES ?
        READ(ITTI,*,ERR=519)IT,NOM
        WRITE(IBA,*)IT,NOM
        SNOM=FLOAT(IT-NOM)

        WRITE(LPT,2112)              ! PARAMETERS FOR CURRENT SCHEME
        WRITE(LPT,2113)IT,NOM,T,T,HSTA,HFIN,HSTE,HX0,HY0,HZ0

        GO TO 243

      END IF

      IF ((ANSRF.EQ.'Y').OR.(ANSRF.EQ.'y')) THEN
      WRITE(I03,509)
      END IF
 1118 WRITE(ITTO,600)                ! MOMENTS IN THE CRYSTAL CELL ?
      READ(ITTI,147)ANSM
      IF ((ANSM.NE.'Y').AND.(ANSM.NE.'y').AND.
     &   (ANSM.NE.'N').AND.(ANSM.NE.'n')) GO TO 1118
      WRITE(IBA,147)ANSM
      IF ((ANSM.EQ.'Y').OR.(ANSM.EQ.'y')) THEN

        CALL PRO_ROTA                ! PROJECTION AND ROTATION OF
                                     ! MOMENTS INSIDE THE CRYSTAL
                                     ! COORDINATES SYSTEM

      END IF

C
C***** OUTPUT OF THE LAST GENERATED SPIN CONFIGURATION *****
C

      SPI=NFIL//'.SPI'
      OPEN(UNIT=I04,STATUS='NEW',FILE=SPI)
      WRITE(I04,2002)TITL1
      WRITE(I04,2002)TITL2

      IF (NQ.NE.0) THEN
        WRITE(I04,2019)NQ,TITL3
      ELSE
        WRITE(I04,2002)TITL3
      END IF

      DO W=1,LW
        DO V=1,LV
          DO U=1,LU

            WRITE(I04,798)
            WRITE(I04,*)U,V,W
            WRITE(I04,799)

            DO N=1,NA

              WRITE(I04,*)N,SX(N,U,V,W),SY(N,U,V,W),SZ(N,U,V,W)

            END DO
          END DO
        END DO
      END DO

C
C ***** OUTPUT COMMENTS AT THE END OF THE PROGRAM *****
C
      WRITE(ITTO,591)NFIL,NFIL,NFIL  ! END COMMENTS
      WRITE(LPT,592)NFIL,NFIL        ! END COMMENTS

      IF(OU3.NE.0) THEN
        WRITE(ITTO,593)NFIL          ! END COMMENTS
        WRITE(LPT,593)NFIL           ! END COMMENTS
      END IF

      WRITE(LPT,597)                 ! END COMMENTS
      STOP


 1233 FORMAT(/,31X,' PROGRAM MCMAG',/,12X,'SIMULATION OF MAGNETIC'
     &       ' STRUCTURE BY A MONTE-CARLO METHOD',/,26X,
     &       'VAX-FORTRAN VERSION 8801',//,
     &       10X,' ENERGY CALCULATED ACCORDING TO THE FOLLOWING'
     &       ' EXPRESSION :',/,14X,
     &       'E = - 0.5*(Si`[Jij]Sj) + ((Di`Si)**2)/|Di| - H`Si',/)
    1 FORMAT(2X,'INPUT FILE NAME (A6) ? :')
    2 FORMAT(A6)
   11 FORMAT(A80)
 2003 FORMAT(1X,' ------------------------------------------'
     &       '----------------------------------',//
     &       ,12X,A50,//,
     &       1X,' ------------------------------------------'
     &       '----------------------------------')
 2004 FORMAT(1X,A80)
   31 FORMAT(1X,' IS THIS THE RIGHT FILE (Y OR N) ? :')
 4001 FORMAT(1X,' ISING (1), XY (2) HEISENBERG (3) OR q-STATE POTTS'
     &       ' PLANAR (4) SPINS ? :')
 4011 FORMAT(1X,' q VALUE (INTEGER) ? :')
 4003 FORMAT(A6,/,'Y',/,I1)
 2001 FORMAT(1X,' TITLE OF SIMULATION (UP TO 50 CHARACTERS) ? :')
 2002 FORMAT(A80)
 2019 FORMAT(I3,A77)
 2220 FORMAT(A50)
 2222 FORMAT(A80/A80/A80)
 2022 FORMAT(2X,A80)
   14 FORMAT(1X,' SIZE OF THE SAMPLE BOX ALONG THE THREE AXES ? :',/
     &       ,1X,' ( MAXIMAL VALUES : ',3(I3,1X),')')
  888 FORMAT(1X,' SAMPLE SIZE MUST BE GREATER THAN OR EQUAL TO 1 ALONG'
     &       ' ANY AXIS')
 7700 FORMAT(1X,' DO YOU WANT TO MODIFY SOME INTERACTIONS IN THE'
     &       ' SAMPLE BOX (Y or N) ? :')
  707 FORMAT(1X,' INITIAL SPIN CONFIGURATION :'/,1X,'RANDOM (R) OR '
     &       'FROM THE INPUT FILE ',A10,' (I) ?')
  708 FORMAT(I1)
  889 FORMAT(I2)
    4 FORMAT(2X,'FREE EDGES (0) PERIODIC (1) OR MIXED BOUNDARY'
     &       ' CONDITIONS (-1) ? : ')
 1262 FORMAT(1X,' GIVE THE INDICES OF 3 SITES WHOSE MOMENTS'
     &       ' WILL BE COLLECTED :')
   41 FORMAT(1X,' GIVE INITIAL TEMPERATURE, HEATING (>1) OR COOLING'
     &       ' (<1) MULTIPLIER AND FINAL TEMPERATURE '
     &       '(IN DEGREE K) :')
  112 FORMAT(1X,' STRENGTH AND DIRECTION OF THE MAGNETIC FIELD ? :')
  501 FORMAT(1X,' GIVE : - TOTAL NUMBER OF MCS/S',/,1X,
     &       '        - NUMBER OF THERMALISATION MCS/S')
  568 FORMAT(1X,' PRINTOUT OF THE CONFIGURATION OF MAGNETIC MOMENTS :',
     &       /,' N = never',/,
     &       ' E = at the final temperature/field only',/,
     &       ' A = at every temperature/field')
 9261 FORMAT(1X,' SCALING OF AVERAGED CHARACTERISTICS OF SAMPLE ? :',/,
     &       ' 1 = per sample',/,
     &       ' 2 = per basic unit cell (input file)',/,
     &       ' 3 = per site',/,
     &       ' 4 = per mole')
  261 FORMAT(1X,' OUTPUT OF AVERAGED CHARACTERISTICS IN FILE ',A10,
     &       ' (Y OR N) ? :')
 1261 FORMAT(1X,' FILE ',A10,' CREATED')
 2008 FORMAT(3X,A80)
 2088 FORMAT(3X,A50)
 2015 FORMAT(10X,'  _________________________________________________'
     &      ,/,11X,'|                                                 |'
     &      ,/,11X,'|        GENERAL PARAMETERS OF SIMULATION         |'
     &      ,/,11X,'|_________________________________________________|'
     &      ,/,11X,'|                                                 |'
     &     ,/,11X,'|  SAMPLE SIZE    :    ',I2,' x',I2,' x',I2,'  CELLS'
     &,10X,'|',/,11X,'|  with  ',I2,' SITES per CELL and ',I2,' MOLES p
     &er CELL  |',/,11X,'|  AVERAGED CHARACTERISTICS   :    per ',A9,
     &   '  |',/,11X,'|  BOUNDARY CONDITIONS        :     ',A10,'    |')
 2014 FORMAT(11X,'|  SPIN MODEL     :     ',I2,A12,'            |'
     &   ,/,11X,'|_________________________________________________|'//)
 2016 FORMAT(11X,'|  SPIN MODEL     :        ',A12,'           |'
     &   ,/,11X,'|_________________________________________________|'//)
 2112 FORMAT(8X,
     & ' _______________________________________________________ '
     &/8X,'|                                                       |'
     &/8X,'|   SIMULATION PARAMETERS USED FOR THE FOLLOWING RUN    |')
 2013 FORMAT(8X,
     & '|_______________________________________________________|'
     &/8X,'|                                                       |'
     &/8X,'| at every T :',I6,'MCS/S  (',I6,' omitted for average) |'
     &/8X,'|     From T =',F8.3,'  to T =',F8.3,'                  |'
     &/8X,'|     Temperature program : T(n+1) = T(n) ',A1,F8.3,'     |'
     &/8X,'|     From H =',F9.2,' to H =',F9.2,'  kOe            |'
     &/8X,'|     Field program       :        none                 |')
 2031 FORMAT(8X,
     &    '|     Field in the direction  :  ',3(F6.2,1X),'  |')
 2030 FORMAT(8X,
     &    '|_______________________________________________________|')
 2113 FORMAT(8X,
     & '|_______________________________________________________|'
     &/8X,'|                                                       |'
     &/8X,'| at any H :',I6,'MCS/S  with',I6,' omitted for average |'
     &/8X,'|     From T =',F8.3,'  to T =',F8.3,'                  |'
     &/8X,'|     Temperature program :        none                 |'
     &/8X,'|     From H =',F9.2,' to H =',F9.2,'  kOe            |'
     &/8X,'|     Field program       : H(n+1) = H(n) + ',F8.2,'    |'
     &/8X,'|     Field in the direction  :  ',3(F6.2,1X),'  |'
     &/8X,'|_______________________________________________________|')
 2018 FORMAT(3X,I3,A77)
 2012 FORMAT(3X,'[ INPUT FILE : ',A6,'.DAT ]  ',A80)
 2009 FORMAT(3X,I2,A13,' SPINS - SAMPLE SIZE : ',I2,' X',I2,' X',I2,' -'
     &       ' RESULTS PER ',A9)
 2099 FORMAT(3X,A13,' SPINS - SAMPLE SIZE : ',I2,' X',I2,' X',I2,' -'
     &       ' RESULTS PER ',A9)
  368 FORMAT(1X,I2,4X,I6,' (-',I5,' )  MCS/S   with H'
     &       ' and/or M// along : ',3(F6.2,2X))
  264 FORMAT(7X,'T',6X,'H',6X,'1/SUS',4X,'SP. HT.',6X,'EN',5X,'MTOT',1X,
     &       3(1X,'M(',I2,')'),5X,'M//',3X,'Fc')
 1494 FORMAT(1X,' STARTING MONTE-CARLO LOOPS')
  494 FORMAT(1X,'***********************',/,' **  T =  ',F8.3,
     &     '    *******************',/,' **  H =',F9.2,
     &     ' kOe ( ',F7.2,' [K] )  **',/,
     &     ' ****************************************')
  484 FORMAT(1X,'***********************',/,' **  T =  ',F8.3,
     &     '    **************************************************',/,
     &     ' **  H =',F9.2,' kOe ( ',F7.2,' [K] )  along : ',3(1X,F6.3),
     &     '  **',/,' ************************************************'
     &     '***********************')
 1018 FORMAT(1X,'!! TEMPERATURE CANNOT BE LESS THAN OR EQ. TO ZERO !!')
  146 FORMAT(1X,' DO YOU WANT TO CONTINUE (Y OR N) ? :')
  147 FORMAT(A1)
  148 FORMAT(1X,' MODIFY TEMPERATURE (T) OR MAGNETIC FIELD (F) ?')
  506 FORMAT(1X,' DEFINE CODE FOR HEATING/COOLING FACTOR :',/,
     &       ' M = multiply',/,
     &       ' A = add')
  606 FORMAT(1X,' HEATING/COOLING FACTOR AND FINAL TEMPERATURE ? :')
  695 FORMAT(//)
  242 FORMAT(1X,' ORIENTATION OF THE MAGNETIC FIELD H ? :')
  241 FORMAT(1X,' STARTING, INCREMENT AND FINAL FIELDS ? :')
  509 FORMAT(' 99')
  600 FORMAT(/,1X,' DO YOU WANT A PRINTOUT OF THE COOMPONENTS '
     &       'OF MAGNETIC MOMENTS',/,'  IN THE CRYSTALLOGRAPHIC SYSTEM'
     &       ' OF COORDINATES ? :')
  798 FORMAT(1X,'CELL')
  799 FORMAT(9X,'Site',5X,'Sx',13X,'Sy',13X,'Sz')
  591 FORMAT(/,' THE FOLLOWING FILES HAVE BEEN CREATED :'
     &      /4X,A6,'.REC    with the sequence of instruction you gave'
     &      /4X,A6,'.LIS    with all the results of simulation'
     &      /4X,A6,'.SPI    with the last generated spin configuration')
  592 FORMAT(/,' THE ADDITIONAL FOLLOWING FILES HAVE BEEN CREATED :'
     &      /4X,A6,'.REC    with the sequence of instruction you gave'
     &      ' for the simulation',
     &      /4X,A6,'.SPI    with the last generated spin configuration')
  593 FORMAT(4X,A6,'.RES    with a condensed list of averaged '
     &       'characteristics')
 3000 FORMAT(1X,' ------------------------------------------'
     &       '----------------------------------')
  596 FORMAT('1','****************************** START SIMULATION **'
     &       '*****************************',//)
  597 FORMAT(///,'************************************ END ********'
     &       '*****************************')
  595 FORMAT(1X)
  495 FORMAT(/)
 1551 FORMAT('1')


      END


      SUBROUTINE READ_FILE

C***********************************************************************
C
C  This subroutine reads the input file MYFILE.DAT which contains the
C  topological and magnetic parameters.
C
C  This subroutine calls no subroutine.
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'
      DIMENSION
     &   JOUR(3),HEURE(2),
     &   AJJX(NAMAX,NNMAX),
     &   AJJY(NAMAX,NNMAX),
     &   AJJZ(NAMAX,NNMAX),
     &   AJXY(NAMAX,NNMAX),
     &   AJXZ(NAMAX,NNMAX),
     &   AJYX(NAMAX,NNMAX),
     &   AJYZ(NAMAX,NNMAX),
     &   AJZX(NAMAX,NNMAX),
     &   AJZY(NAMAX,NNMAX),
     &   AMPS(NAMAX),
     &   PT(NNMAX),
     &   NAT(NAMAX),
     &   NVS(NAMAX),
     &   NAV(NAMAX,NNMAX),
     &   M(NAMAX,NNMAX,3),
     &   D0(NAMAX),DX(NAMAX),DY(NAMAX),DZ(NAMAX),
     &   D(NAMAX),DDIX(NAMAX),DDIY(NAMAX),DDIZ(NAMAX)


      INTEGER NA,JCOD,NMOT,NAV,NAT,NVS,M,NIN,NFI

      CHARACTER TITL1*80,TITL2*80,NFIL*6,NFILA*10

      COMMON
     &       /COM4/M
     &       /COM2/NA,IANI
     &       /COM8/NVS
     &       /TEST/NAV,AJJX,AJJY,AJJZ,AJXY,AJXZ,AJYZ,
     &             AJYX,AJZX,AJZY,PT,NAT
     &       /TEST2/JCOD
     &       /READ4/D,DX,DY,DZ
     &       /READ5/DDIX,DDIY,DDIZ,D0
     &       /READ1/NMOT
     &       /READ2/TITL1,TITL2,NFIL
     &       /COM5/AMPS

      CALL DATE(JOUR)
      CALL TIME(HEURE)
      WRITE(LPT,13)JOUR(1),JOUR(2),JOUR(3),HEURE(1),HEURE(2)
      WRITE(LPT,3000)
      WRITE(LPT,1233)
      WRITE(LPT,3000)
      WRITE(LPT,2011)NFIL
      WRITE(LPT,25)TITL1
      WRITE(LPT,25)TITL2
      WRITE(LPT,12)
      NFILA = NFIL//'.DAT'
      READ(I01,*,ERR=1121)NA,JCOD,NMOT

      IF (((JCOD.NE.0).AND.(JCOD.NE.1).AND.(JCOD.NE.2)).
     &    OR.(NA.EQ.0).OR.(NMOT.LT.1)) THEN
 1121   WRITE (ITTO,1122)NFILA
        WRITE (ITTO,1123)
        STOP
      END IF

      IANI=1

      IF (NA.LT.0) THEN
        IANI=-1
        NA=-NA
      END IF


      DO I=1,NA

        IF (IANI.EQ.-1) THEN
          READ(I01,*,ERR=1124)NAT(I),NVS(NAT(I))
        ELSE
          READ(I01,*,ERR=1124)NAT(I),NVS(NAT(I)),D(NAT(I)),
     &                        DDIX(NAT(I)),DDIY(NAT(I)),DDIZ(NAT(I))
        END IF
        IF ((NAT(I).LT.1).OR.(NVS(NAT(I)).LT.1).OR.
     &      (NAT(I).GT.NA)) THEN
 1124     WRITE (ITTO,1122)NFILA
          WRITE (ITTO,1125)
          STOP
        END IF

        WRITE(LPT,24)NAT(I),D(NAT(I)),
     &               DDIX(NAT(I)),DDIY(NAT(I)),DDIZ(NAT(I)),
     &               NVS(NAT(I))


        IF (JCOD.EQ.0) THEN          ! JCOD=0 => ISOTROPIC EXCHANGE
          DO J=1,NVS(NAT(I))
            READ(I01,*,ERR=1126)NAV(NAT(I),J),(M(NAT(I),J,K),K=1,3),
     &                          AJJX(NAT(I),J)
            IF ((NAV(NAT(I),J).LT.1).OR.(NAV(NAT(I),J).GT.NA)) THEN
 1126         WRITE (ITTO,1122)NFILA
              WRITE (ITTO,1127)
              STOP
            END IF
            WRITE(LPT,17)NAV(NAT(I),J),(M(NAT(I),J,K),K=1,3),
     &                   NAT(I),NAV(NAT(I),J),AJJX(NAT(I),J)
            AJJY(NAT(I),J)=AJJX(NAT(I),J)
            AJJZ(NAT(I),J)=AJJX(NAT(I),J)
          END DO


        ELSE IF (JCOD.EQ.1) THEN     ! JCOD=1 => DIAGONAL TENSOR
          DO J=1,NVS(NAT(I))
            READ(I01,*,ERR=1128)NAV(NAT(I),J),(M(NAT(I),J,K),K=1,3),
     &                 AJJX(NAT(I),J),AJJY(NAT(I),J),AJJZ(NAT(I),J)
            IF (NAV(NAT(I),J).LT.1) THEN
 1128         WRITE (ITTO,1122)NFILA
              WRITE (ITTO,1127)
              STOP
            END IF
            WRITE(LPT,92)NAV(NAT(I),J),(M(NAT(I),J,K),K=1,3),
     &                   NAT(I),NAV(NAT(I),J),
     &                   AJJX(NAT(I),J),AJJY(NAT(I),J),AJJZ(NAT(I),J)
          END DO


        ELSE IF (JCOD.EQ.2) THEN     ! JCOD=2 => ANISOTROPIC TENSOR
          DO J=1,NVS(NAT(I))
            READ(I01,*,ERR=1129)NAV(NAT(I),J),(M(NAT(I),J,K),K=1,3),
     &                 AJJX(NAT(I),J),AJJY(NAT(I),J),AJJZ(NAT(I),J),
     &                 AJXY(NAT(I),J),AJXZ(NAT(I),J),AJYZ(NAT(I),J),
     &                 AJYX(NAT(I),J),AJZX(NAT(I),J),AJZY(NAT(I),J)
            IF (NAV(NAT(I),J).LT.1) THEN
 1129         WRITE (ITTO,1122)NFILA
              WRITE (ITTO,1127)
              STOP
            END IF
            WRITE(LPT,192)NAV(NAT(I),J),(M(NAT(I),J,K),K=1,3),
     &                    NAT(I),NAV(NAT(I),J),
     &                    AJJX(NAT(I),J),AJXY(NAT(I),J),AJXZ(NAT(I),J),
     &                    AJYX(NAT(I),J),AJJY(NAT(I),J),AJYZ(NAT(I),J),
     &                    AJZX(NAT(I),J),AJZY(NAT(I),J),AJJZ(NAT(I),J)
          END DO

        END IF

      END DO

      WRITE(LPT,695)

   27 READ(I01,*,ERR=1130)NIN,NFI,SMO

      IF ((NIN.LT.1).OR.(NFI.LT.1).OR.(NFI.LT.NIN).OR.
     &    (SMO.LT.0.)) THEN
 1130   WRITE (ITTO,1122)NFILA
        WRITE (ITTO,1131)
        STOP
      END IF

      NACO = NACO+NFI-NIN+1

      DO I=NIN,NFI
        AMPS(I)=SMO
        WRITE(LPT,9878)I,AMPS(I)
      END DO

      IF (NACO.NE.NA) GO TO 27

      DO I=1,NA
        D0(I)=SQRT(DDIX(I)*DDIX(I)+DDIY(I)*DDIY(I)+DDIZ(I)*DDIZ(I))
        IF(D0(I).NE.0) THEN
          DX(I)=DDIX(I)/D0(I)
          DY(I)=DDIY(I)/D0(I)
          DZ(I)=DDIZ(I)/D0(I)
        END IF
      END DO

      WRITE(ITTO,595)

 
   13 FORMAT(/,29X,3A4,2X,2A4//)
 1122 FORMAT(/,1X,' VERIFY YOUR INPUT FILE ',A10,/,
     &       '         POSSIBLE ERRORS :')
 1123 FORMAT(1X,' NA and NMOT must be integers and at least equal to 1'
     &       ,/,1X,' JCOD must be equal to 0, 1 or 2')
 1125 FORMAT(1X,' NAT must be an integer between 1 and NA'
     &       ,/, 1X,' NV must be an integer at least equal to 1')
 1127 FORMAT(1X,' NAV must be an integer between 1 and NA'
     &       ,/,1X,' Cell indices must be integers')
 1131 FORMAT(1X,' NI and NF must be integers and at least equal to 1'
     &       ,/,1X,' NF must always be larger than or equal to NI'
     &       ,/,1X,' The largest NF must be equal to NAT'
     &       ,/,1X,' A spin amplitude must be attributed to any site'
     &       ,/,1X,' Spin amplitudes must be positive or equal to 0')
 1233 FORMAT(/,31X,' PROGRAM MCMAG',/,12X,'SIMULATION OF MAGNETIC'
     &       ' STRUCTURE BY A MONTE-CARLO METHOD',/,26X,
     &       'VAX-FORTRAN VERSION 8801',//,
     &       10X,' ENERGY CALCULATED ACCORDING TO THE FOLLOWING'
     &       ' EXPRESSION :',/,14X,
     &       'E = - 0.5*(Si`[Jij]Sj) + ((Di`Si)**2)/|Di| - H`Si',/)
 2011 FORMAT(//,1X,' Input file : ',A6,'.DAT  contains the '
     &       'following data : ',//)
   12 FORMAT(/)
   25 FORMAT(1X,A80)
   17 FORMAT(9X,'Site ',I2,'  in cell (',I2,',',I2,',',I2,')',
     &       '  with J(',I2,',',I2,') =',F10.3,' K')
   24 FORMAT(/,1X,'SITE ',I2,' [ D = ',F8.3,' along the direction ',
     &       3(F6.3,2X),']',/,9X,'with ',I2,' neighbours :')
   92 FORMAT(9X,'Site ',I2,'  in cell (',I2,',',I2,',',I2,')',
     &       '  with coupling (',I2,',',I2,') :',/,15X,
     &       'Jx = ',F8.3,' K , Jy = ',F8.3,' K , Jz = ',F8.3,' K')
 9878 FORMAT(22X,' Spin amplitude ( ',I2,' ) = ',F5.3)
  192 FORMAT(9X,'Site ',I2,'  in cell (',I2,',',I2,',',I2,')',/,
     &       15X,'with J tensor (',I2,',',I2,') =   ',3(F8.3,X),/,
     &       41X,3(F8.3,X),/,41X,3(F8.3,X))
 9876 FORMAT(1X,'D(',I2,') = ',F8.3,2X,'( direction ',3(F6.3,2X),
     &        ')')
 3000 FORMAT(1X,' ------------------------------------------'
     &       '----------------------------------')
  695 FORMAT(//)
  595 FORMAT(1X)
  147 FORMAT(A1)

      RETURN

      END


      SUBROUTINE COMP_TEST


C***********************************************************************
C
C  This subroutine tests the validity of connectivity between spins.
C  Every interaction is described twice in the input file (spin i with 
C  neighbour j, then spin j with neighbour i).
C  The subroutine checks that the two descriptions are compatible
C  as well from the topological viewpoint as from the coupling
C  constant viewpoint.
C  Any time an anomaly is detected, an error message is generated,
C  that gives the location of the corresponding error.
C
C  The test is designed as follows :
C  - TOPOLOGY
C    For any coupling Jij, the consistency of the cell relative
C    indexing (M(i,j,k), k=1,2,3) of neighbouring spins is checked
C    by calculating TCO(k) = M(i,j,k) + M(j,i,k). If any anomaly is
C    detected (non-zero TCO(k)), an error message is generated, that
C    gives the location of corresponding error (namely M(i,j,k))
C  - INTERACTIONS
C    Following the same scheme, the consistency of interactions Jij
C    is checked by calculating TCOJ* = Jij - Jji for any component
C    of the coupling tensor (* means X,Y,Z,XY,XZ,YX,YZ,ZX or ZY).
C    If any anomaly is detected (non-zero TCOJ*), an error message
C    is generated, that gives the location of corresponding error
C    (namely J(i,j))
C  Remark : For specific purpose, such as the study of non symmetrical
C  coupling (Jij different from Jji), the last part of the test (at 
C  least) should be bypassed. If you bypass the test, make sure that
C  your input data are correct !
C
C  This subroutine calls no subroutine.
C
C***********************************************************************

      INCLUDE 'MCMAG.INC'
      DIMENSION
     &   AJJX(NAMAX,NNMAX),
     &   AJJY(NAMAX,NNMAX),
     &   AJJZ(NAMAX,NNMAX),
     &   AJXY(NAMAX,NNMAX),
     &   AJXZ(NAMAX,NNMAX),
     &   AJYX(NAMAX,NNMAX),
     &   AJYZ(NAMAX,NNMAX),
     &   AJZX(NAMAX,NNMAX),
     &   AJZY(NAMAX,NNMAX),
     &   PT(NNMAX),
     &   NAT(NAMAX),
     &   NVS(NAMAX),
     &   NAV(NAMAX,NNMAX),
     &   M(NAMAX,NNMAX,3),
     &   TCO(3)

      REAL*8 TCO,TCOJX,TCOJY,TCOJZ,
     &       TCOJXY,TCOJXZ,TCOJYZ,TCOJYX,TCOJZX,TCOJZY

      COMMON 
     &       /COM4/M
     &       /COM2/NA
     &       /COM8/NVS
     &       /TEST/NAV,AJJX,AJJY,AJJZ,AJXY,AJXZ,AJYZ,
     &       AJYX,AJZX,AJZY,PT,NAT,TCO,TCOJX,TCOJY,TCOJZ,
     &       TCOJXY,TCOJXZ,TCOJYZ,TCOJYX,TCOJZX,TCOJZY
     &       /TEST2/JCOD

      NH=1
      NHP=1

      DO I=1,NA

        DO NJ=1,NNMAX
          PT(NJ)=0
        END DO

        DO 106 J=1,NVS(NAT(I))
          IF((JCOD.EQ.2).AND.(NAT(I).EQ.NAV(NAT(I),J)))GO TO 4446
          DO NI=1,NNMAX
            IF ( NAV(NAT(I),J).EQ.PT(NI) ) GO TO 106
          END DO
          DO K=1,3
            TCO(K)=M(NAT(I),J,K)
          END DO
          TCOJX=AJJX(NAT(I),J)
          IF (JCOD.GT.0) THEN
            TCOJY=AJJY(NAT(I),J)
            TCOJZ=AJJZ(NAT(I),J)
            IF (JCOD.EQ.2) THEN
              TCOJXY=AJXY(NAT(I),J)
              TCOJXZ=AJXZ(NAT(I),J)
              TCOJYX=AJYX(NAT(I),J)
              TCOJYZ=AJYZ(NAT(I),J)
              TCOJZX=AJZX(NAT(I),J)
              TCOJZY=AJZY(NAT(I),J)
            END IF
          END IF

          NH=1

          DO N=J+1,NVS(NAT(I))
            IF ( NAV(NAT(I),N).EQ.NAV(NAT(I),J) ) THEN
              DO K=1,3
                TCO(K)=TCO(K)+M(NAT(I),N,K)
              END DO
              NH=-NH
              IF(NAV(NAT(I),N).NE.NAT(I))NH=1
              TCOJX=TCOJX+NH*AJJX(NAT(I),N)
              IF (JCOD.GT.0) THEN
                TCOJY=TCOJY+NH*AJJY(NAT(I),N)
                TCOJZ=TCOJZ+NH*AJJZ(NAT(I),N)
                IF (JCOD.EQ.2) THEN
                  TCOJXY=TCOJXY+NH*AJXY(NAT(I),N)
                  TCOJXZ=TCOJXZ+NH*AJXZ(NAT(I),N)
                  TCOJYX=TCOJYX+NH*AJYX(NAT(I),N)
                  TCOJYZ=TCOJYZ+NH*AJYZ(NAT(I),N)
                  TCOJZX=TCOJZX+NH*AJZX(NAT(I),N)
                  TCOJZY=TCOJZY+NH*AJZY(NAT(I),N)
                END IF
              END IF
              PT(N)=NAV(NAT(I),N)
            END IF
          END DO
          II=NAV(NAT(I),J)
          NHP=1

          DO L=1,NVS(II)
            IF ( NAV(II,L).EQ.NAT(I) ) THEN
              DO K=1,3
                TCO(K)=TCO(K)+M(NAV(NAT(I),J),L,K)
              END DO
              NHP=-NHP
              IF (II.NE.NAT(I)) NHP=1
                TCOJX=TCOJX-NHP*AJJX(NAV(NAT(I),J),L)
              IF (JCOD.GT.0) THEN
                TCOJY=TCOJY-NHP*AJJY(NAV(NAT(I),J),L)
                TCOJZ=TCOJZ-NHP*AJJZ(NAV(NAT(I),J),L)
                IF (JCOD.GT.0) THEN
                  TCOJXY=TCOJXY-NHP*AJYX(NAV(NAT(I),J),L)
                  TCOJXZ=TCOJXZ-NHP*AJZX(NAV(NAT(I),J),L)
                  TCOJYX=TCOJYX-NHP*AJXY(NAV(NAT(I),J),L)
                  TCOJYZ=TCOJYZ-NHP*AJZY(NAV(NAT(I),J),L)
                  TCOJZX=TCOJZX-NHP*AJXZ(NAV(NAT(I),J),L)
                  TCOJZY=TCOJZY-NHP*AJYZ(NAV(NAT(I),J),L)
                END IF
              END IF
            END IF
          END DO

       	  DO K=1,3
            IF (JCOD.EQ.0) THEN
              IF ( (TCO(K).NE.0).OR.(TCOJX.NE.0) ) THEN
                WRITE(LPT,131)
                WRITE(ITTO,131)
                WRITE(LPT,109)NAT(I),NAV(NAT(I),J),K,
     &                        TCO(K),NAT(I),NAV(NAT(I),J),
     &                        TCOJX,NAT(I),NAV(NAT(I),J)
                WRITE(ITTO,109)NAT(I),NAV(NAT(I),J),K,
     &                        TCO(K),NAT(I),NAV(NAT(I),J),
     &                        TCOJX,NAT(I),NAV(NAT(I),J)
                GO TO 711
              END IF

            ELSE IF (JCOD.EQ.1) THEN
              IF ( (TCO(K).NE.0).OR.(TCOJX.NE.0).OR.
     &            (TCOJY.NE.0).OR.(TCOJZ.NE.0) ) THEN
                WRITE(LPT,131)
                WRITE(ITTO,131)
                WRITE(LPT,709)NAT(I),NAV(NAT(I),J),K,
     &                        TCO(K),NAT(I),NAV(NAT(I),J),
     &                        TCOJX,NAT(I),NAV(NAT(I),J),
     &                        TCOJY,NAT(I),NAV(NAT(I),J),
     &                        TCOJZ,NAT(I),NAV(NAT(I),J)
                WRITE(ITTO,709)NAT(I),NAV(NAT(I),J),K,
     &                        TCO(K),NAT(I),NAV(NAT(I),J),
     &                        TCOJX,NAT(I),NAV(NAT(I),J),
     &                        TCOJY,NAT(I),NAV(NAT(I),J),
     &                        TCOJZ,NAT(I),NAV(NAT(I),J)
                GO TO 711
              END IF

            ELSE IF (JCOD.EQ.2) THEN
              IF ( (TCO(K).NE.0).OR.(TCOJX.NE.0).OR.
     &             (TCOJY.NE.0).OR.(TCOJZ.NE.0).OR.
     &             (TCOJXY.NE.0).OR.(TCOJXZ.NE.0).OR.
     &             (TCOJYX.NE.0).OR.(TCOJYZ.NE.0).OR.
     &             (TCOJZX.NE.0).OR.(TCOJZY.NE.0) ) THEN
                WRITE(LPT,131)
                WRITE(ITTO,131)
                WRITE(LPT,809)NAT(I),NAV(NAT(I),J),K,
     &                        TCO(K),NAT(I),NAV(NAT(I),J),
     &                        TCOJX,TCOJXY,TCOJXZ,NAT(I),NAV(NAT(I),J),
     &                        TCOJYX,TCOJY,TCOJYZ,NAT(I),NAV(NAT(I),J),
     &                        TCOJZX,TCOJZY,TCOJZ,NAT(I),NAV(NAT(I),J)
                WRITE(ITTO,809)NAT(I),NAV(NAT(I),J),K,
     &                        TCO(K),NAT(I),NAV(NAT(I),J),
     &                        TCOJX,TCOJXY,TCOJXZ,NAT(I),NAV(NAT(I),J),
     &                        TCOJYX,TCOJY,TCOJYZ,NAT(I),NAV(NAT(I),J),
     &                        TCOJZX,TCOJZY,TCOJZ,NAT(I),NAV(NAT(I),J)
                GO TO 711
              END IF
            END IF
          END DO

  106   END DO

      END DO

      WRITE(LPT,132)
      WRITE(ITTO,133)
      GO TO 811
 4446 WRITE(ITTO,4447)

  711 STOP


  131 FORMAT(///1X,' ------------------------------------------'
     &       '----------------------------------',
     &       //,24X,' COMPATIBILITY TEST : BAD !!',/,20X,'******'
     &       ' VERIFY YOUR DATA FILE ! ******',//,
     &       1X,' ------------------------------------------'
     &       '----------------------------------')
  132 FORMAT(///1X,' ------------------------------------------'
     &       '----------------------------------',//
     &       ,20X,' COMPATIBILITY TEST : DATA FILE LOOKS OK',//,
     &       1X,' ------------------------------------------'
     &       '----------------------------------')
  133 FORMAT(1X,' ------------------------------------------'
     &       '----------------------------------',//
     &       ,20X,' COMPATIBILITY TEST : DATA FILE LOOKS OK',//,
     &       1X,' ------------------------------------------'
     &       '----------------------------------')
  109 FORMAT(1X,'  TEST SYM. ON  M (',3I2,' ) = ',F5.2,/,
     &       '     TEST ON J (',2I2,' ) = ',F8.3,/,
     &       ' Dubious connectivity or exchange constant between site '
     &       ,I2,' and site ',I2,/)
  709 FORMAT(1X,' TEST SYM. ON  M (',3I2,' ) = ',F5.2,/,
     &       '    TEST ON JX (',2I2,' ) = ',F8.3,/,
     &       '    TEST ON JY (',2I2,' ) = ',F8.3,/,
     &       '    TEST ON JZ (',2I2,' ) = ',F8.3,/,
     &       ' Dubious connectivity or exchange constant between site '
     &       ,I2,' and site ',I2,/)
  809 FORMAT(1X,' TEST SYM. ON  M (',3I2,' ) = ',F5.2,/,
     &       '    TEST ON JXX,JXY,JXZ (',2I2,' ) = ',3(F8.3,X),/,
     &       '    TEST ON JYX,JYY,JYZ (',2I2,' ) = ',3(F8.3,X),/,
     &       '    TEST ON JZX,JZY,JZZ (',2I2,' ) = ',3(F8.3,X),/,
     &       ' Dubious connectivity or exchange constant between site '
     &       ,I2,' and site ',I2,/)
 4447 FORMAT(1X,' PLEASE DON"T CONNECT SITES TO THEMSELVES WITH '
     &       'ASYMMETRIC COUPLING',/,'  YOU SHOULD DOUBLE YOUR CELL',/)

  811 RETURN

      END


      SUBROUTINE MOD_INT

C***********************************************************************
C
C  This subroutine allows to modify one or more coupling constant(s)
C  within the sample box, for instance to test the perturbation of
C  the magnetic configuration due to the introduction of impurities
C  in the sample. The subroutine is activated only if JCOD=0
C  (isotropic exchange).
C
C  This subroutine calls no subroutine.
C
C***********************************************************************

      INCLUDE 'MCMAG.INC'
      DIMENSION
     &   NVS(NAMAX),
     &   NAV(NAMAX,NNMAX),
     &   M(NAMAX,NNMAX,3),
     &   AJJX(NAMAX,NNMAX),
     &   IUM(NMOMAX),IVM(NMOMAX),IWM(NMOMAX),
     &   NM(NMOMAX),NUMB(NMOMAX),AJMOD(NMOMAX)


      COMMON 
     &       /PALL/LU,LV,LW
     &       /COM4/M
     &       /COM8/NVS
     &       /TEST/NAV,AJJX
     &       /CYC2/AJMOD,IUM,IVM,IWM,NM,NUMB,IM


      IM=1
 7706 WRITE(ITTO,7701)
      READ(ITTI,*)NM(IM),IUM(IM),IVM(IM),IWM(IM)
      WRITE(IBA,*)NM(IM),IUM(IM),IVM(IM),IWM(IM)

      IF (NM(IM).NE.0) THEN
        WRITE(ITTO,7702)NM(IM),NVS(NM(IM))
        DO J=1,NVS(NM(IM))
          WRITE(ITTO,7703)J,NAV(NM(IM),J),(M(NM(IM),J,K),K=1,3),
     &                    AJJX(NM(IM),J)
        END DO
 7708   WRITE(ITTO,7705)
        READ(ITTI,*)NUMB(IM),AJMOD(IM)
        WRITE(IBA,*)NUMB(IM),AJMOD(IM)
        IF(NUMB(IM).EQ.0)GO TO 7706
        JA=NAV(NM(IM),NUMB(IM))

        DO JB=1,NVS(JA)
          MAJ1=M(JA,JB,1)+M(NM(IM),NUMB(IM),1)
          MAJ2=M(JA,JB,2)+M(NM(IM),NUMB(IM),2)
          MAJ3=M(JA,JB,3)+M(NM(IM),NUMB(IM),3)
          IF ( (NAV(JA,JB).EQ.NM(IM)).AND.(MAJ1.EQ.0).AND.
     &         (MAJ2.EQ.0).AND.(MAJ3.EQ.0) ) THEN
            JBA=JB
          END IF
        END DO

        NUMB(IM+1)=JBA
        AJMOD(IM+1)=AJMOD(IM)
        NM(IM+1)=JA
        IUM(IM+1)=IUM(IM)+M(NM(IM),NUMB(IM),1)
        IF(IUM(IM+1).LE.0) IUM(IM+1)=IUM(IM+1)+LU
        IF(IUM(IM+1).GT.LU) IUM(IM+1)=IUM(IM+1)-LU
        IVM(IM+1)=IVM(IM)+M(NM(IM),NUMB(IM),2)
        IF(IVM(IM+1).LE.0) IVM(IM+1)=IVM(IM+1)+LV
        IF(IVM(IM+1).GT.LV) IVM(IM+1)=IVM(IM+1)-LV
        IWM(IM+1)=IWM(IM)+M(NM(IM),NUMB(IM),3)
        IF(IWM(IM+1).LE.0) IWM(IM+1)=IWM(IM+1)+LW
        IF(IWM(IM+1).GT.LW) IWM(IM+1)=IWM(IM+1)-LW
        IM=IM+2
        NM(IM)=NM(IM-2)
        IUM(IM)=IUM(IM-2)
        IVM(IM)=IVM(IM-2)
        IWM(IM)=IWM(IM-2)
        GO TO 7708

      END IF

      IM=IM-1
      WRITE(ITTO,7711)
      WRITE(LPT,7711)
      DO I=1,IM
        WRITE(ITTO,7712)NM(I),IUM(I),IVM(I),IWM(I),NUMB(I),AJMOD(I)
        WRITE(LPT,7712)NM(I),IUM(I),IVM(I),IWM(I),NUMB(I),AJMOD(I)
      END DO
      WRITE(ITTO,495)
      WRITE(LPT,495)

 7701 FORMAT(1X,' IDENTIFY SITE (N) AND CELL (U,V,W) [0 0 0 0'
     &       ' TO STOP] :')
 7702 FORMAT(1X,' SITE NUMBER',I2,' HAS ',I2,' NEIGHBOURS :')
 7703 FORMAT(1X,' NEIGH. NUMB.',I2,' = SITE',I2,' IN RELATIVE CELL'
     &       ' (',I2,',',I2,',',I2,') WITH J =',F8.3,' K')
 7705 FORMAT(1X,' ENTER MODIFICATION (NEIGH. NUMB. and NEW J) [0 0'
     &       ' TO STOP] :')
 7711 FORMAT(/,1X,' THE FOLLOWING MODIFICATIONS HAVE BEEN APPLIED'
     &       ' FOR THE CURRENT SIMULATION :',/,1X,'(ORIGINAL FILE IS'
     &       ' PRESERVED)')
 7712 FORMAT(1X,' FROM SITE',I2,' IN CELL (',I2,',',I2,',',I2,')'
     &       ' TO ITS NEIGH. NUMB.',I2,' : NEW J =',F8.3,' K')
  495 FORMAT(/)

      RETURN

      END


      SUBROUTINE GEN_SPI

C***********************************************************************
C
C  This subroutine generates or reads the input spin configuration.
C  In the former case, spins are randomly generated.
C  In the latter case, spin values are read from the file MYFILE.INI.
C  The input format of this file is identical to the format of the
C  file MYFILE.SPI which is generated at the end of every simulation and
C  contains the last generated spin configuration of the simulation. Thus
C  a simulation can be split up in several runs.
C
C  This subroutine calls the following subroutines:
C     RANnD(RADIUS,X1,Y1...)
C
C***********************************************************************

      INCLUDE 'MCMAG.INC'

      DIMENSION
     &   AMPS(NAMAX),
     &   SX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   D(NAMAX),DX(NAMAX),DY(NAMAX),DZ(NAMAX)


      INTEGER U,V,W

      CHARACTER INI*10

      COMMON
     &       /PALL/LU,LV,LW,IRAN
     &       /COM2/NA,IANI
     &       /COM8/NVS
     &       /CYC3/SX,SY,SZ
     &       /COM5/AMPS,AMPS2
     &       /READ3/INI,IANS
     &       /READ4/D,DX,DY,DZ
     &       /WRIT/IWMX,IWMXY


      IF(IRAN.EQ.1) THEN
C
C*****     FOR ISING SPINS     *****
C
        DO W=1,LW
          DO V=1,LV
            DO U=1,LU

              IF ((IANS.EQ.'I').OR.(IANS.EQ.'i')) THEN 
                READ(I02,*)
                READ(I02,*)JU,JV,JW
                READ(I02,*)
              END IF

              DO N=1,NA
                IF ((IANS.EQ.'R').OR.(IANS.EQ.'r')) THEN 
                  CALL RAN1D (AMPS(N),X,Y,Z,N)
                  SX(N,U,V,W)=X
                  SY(N,U,V,W)=Y
                  SZ(N,U,V,W)=Z
                ELSE
                  IF (IANI.EQ.-1) THEN
                    READ(I02,*)JN,SX(JN,JU,JV,JW)
                  ELSE
                    READ(I02,*)JN,SX(JN,JU,JV,JW),SY(JN,JU,JV,JW),
     &                         SZ(JN,JU,JV,JW)
                  END IF
                END IF
              END DO

            END DO
          END DO
        END DO


      ELSE IF(IRAN.EQ.2) THEN
C
C*****     FOR XY SPINS     *****
C
        DO W=1,LW
          DO V=1,LV
            DO U=1,LU

              IF ((IANS.EQ.'I').OR.(IANS.EQ.'i')) THEN 
                READ(I02,*)
                READ(I02,*)JU,JV,JW
                READ(I02,*)
              END IF

              DO N=1,NA
                IF ((IANS.EQ.'R').OR.(IANS.EQ.'r')) THEN 
                  CALL RAN2D (AMPS(N),X,Y)
                  SX(N,U,V,W)=X
                  SY(N,U,V,W)=Y
                ELSE
                  READ(I02,*)JN,SX(JN,JU,JV,JW),SY(JN,JU,JV,JW)
                END IF
              END DO

            END DO
          END DO
        END DO


      ELSE IF(IRAN.EQ.3) THEN
C
C*****     FOR HEISENBERG SPINS     *****
C
        DO W=1,LW
          DO V=1,LV
            DO U=1,LU

              IF ((IANS.EQ.'I').OR.(IANS.EQ.'i')) THEN 
                READ(I02,*)
                READ(I02,*)JU,JV,JW
                READ(I02,*)
                WRITE(ITTO,*)JU,JV,JW
              END IF

              DO N=1,NA
                IF ((IANS.EQ.'R').OR.(IANS.EQ.'r')) THEN 
                  CALL RAN3D (AMPS(N),X,Y,Z)
                  SX(N,U,V,W)=X
                  SY(N,U,V,W)=Y
                  SZ(N,U,V,W)=Z
                ELSE
                  READ(I02,*)JN,SX(JN,JU,JV,JW),SY(JN,JU,JV,JW),
     &                       SZ(JN,JU,JV,JW)
                END IF
              END DO

            END DO
          END DO
        END DO


      ELSE IF (IRAN.EQ.4) THEN
C
C*****     FOR q-STATE POTTS PLANAR SPINS     *****
C
        DO W=1,LW
          DO V=1,LV
            DO U=1,LU

              IF ((IANS.EQ.'I').OR.(IANS.EQ.'i')) THEN 
                READ(I02,*)
                READ(I02,*)JU,JV,JW
                READ(I02,*)
              END IF

              DO N=1,NA
                IF ((IANS.EQ.'R').OR.(IANS.EQ.'r')) THEN 
                  CALL RANqD (AMPS(N),X,Y)
                  SX(N,U,V,W)=X
                  SY(N,U,V,W)=Y
                ELSE
                  READ(I02,*)JN,SX(JN,JU,JV,JW),SY(JN,JU,JV,JW)
                END IF
              END DO

            END DO
          END DO
        END DO


      END IF

C
C*****     END OF GENERATION AND READING     *****
C
C
C*****     OUTPUT     *****
C
      IF ((IANS.EQ.'R').OR.(IANS.EQ.'r')) THEN 
        WRITE(LPT,705)
      ELSE
        WRITE(LPT,602)INI
      END IF

      DO W=1,LW
        DO V=1,LV
          DO U=1,LU

            NPI=1
            NPF=NA
            WRITE(LPT,598)U,V,W
            WRITE(ITTO,598)U,V,W
            WRITE(LPT,499)
            WRITE(ITTO,499)

            NPASSI=0
 4412       IF((NA-NAL*NPASSI).GT.NAL) NPF=NPI+NAL-1
            NPASSI=NPASSI+1
            WRITE(ITTO,491)
            WRITE(LPT,491)
            DO N=NPI,NPF
              WRITE(ITTO,496)N
              WRITE(LPT,496)N
            END DO
            WRITE(LPT,497)(SX(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,497)(SX(N,U,V,W),N=NPI,NPF)
            IF (IWMX.EQ.1) GO TO 999
            WRITE(LPT,493)(SY(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,493)(SY(N,U,V,W),N=NPI,NPF)
            IF (IWMXY.EQ.1) GO TO 999
            WRITE(LPT,494)(SZ(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,494)(SZ(N,U,V,W),N=NPI,NPF)
  999       WRITE(LPT,595)
            WRITE(ITTO,595)
            IF (NPF.GE.NPI+NAL-1) THEN
              IF ((NPI.NE.1).OR.(NPF.NE.NA)) THEN
                NPI=NPF+1
                NPF=NA
                GO TO 4412
              END IF
            END IF

          END DO
        END DO
      END DO


  496 FORMAT('+Site',I2,'   ',$)
  497 FORMAT(1X,' Sx ',50(1X,F6.3,2X))
  493 FORMAT(1X,' Sy ',50(1X,F6.3,2X))
  494 FORMAT(1X,' Sz ',50(1X,F6.3,2X))
  491 FORMAT('+     ',$)
  499 FORMAT()
  705 FORMAT(1X,///,1X,' RANDOM INITIAL SPIN CONFIGURATION ',//)
  602 FORMAT(1X,//,1X,' INITIAL SPIN CONFIGURATION READ '
     &       'FROM FILE ',A10)
 2002 FORMAT(2X,A80,/)
  598 FORMAT(1X,' CELL (',I2,',',I2,',',I2,')')
  595 FORMAT(/)

      RETURN

      END


	SUBROUTINE MC_CYCLE

C***********************************************************************
C
C  This subroutine executes the Monte-Carlo loops according to the
C  following scheme:
C
C
C               |
C               V
C               |
C               |--------->--------.         
C               |                  |         
C               |---->---.         |         
C               |        |         |         
C               |   -----------    |         
C               |  | spin loop |   |         
C               |  | on sample |   |         
C               |   -----------    |         
C               |        |         |         
C               |----<---'    -----------    
C               |            | IT loops  |   
C               |            | of MCS/S  |   
C               |             -----------    
C               |                  |         
C               |---------<--------'
C               |
C               V
C               |
C
C
C  This subroutine calls the following subroutines and function:
C  ENn_CALC(N,U,V,W)
C  RANF() : this portable real function [Program library of CERN 
C           Computer Centre (CERN - Geneva)] is used to generate random 
C           numbers between 0 and 1. This function may be slower than
C           dedicated random number generators. Therefore, in order to 
C           save CPU time, users should replace RANF() by a call to
C           the equivalent function on the computer they use.
C           For instance, DEC-computer users should call the random 
C           generator RAN(I1,I2). For sake of convenience, the calls to
C           this function are included in the current version of the
C           program as comment cards. After implementing and testing
C           the program (see manual), the use of RAN(I1,I2) or of a
C           faster computer random generator, is recommended.
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'


      DIMENSION
     &   SX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMX(NAMAX,NUMAX,NVMAX,NWMAX),
     &   COMX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMX2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMY2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMZ2(NAMAX,NUMAX,NVMAX,NWMAX)


      INTEGER U,V,W

      INTEGER*2 I1,I2


      REAL*8 ENM,EN2M,MAG2,
     &       EN,ENMIN


      COMMON
     &       /PALL/LU,LV,LW,IRAN
     &       /COM2/NA
     &       /CYC8/DELTA,X,Y,Z
     &       /CYC9/SOMX
     &       /CYC10/IT,NOM
     &       /SOMM2/COMX
     &       /CYC7/ENM,EN2M,MAG2
     &       /COM6/NPAJ
     &       /CYC3/SX,SY,SZ
     &       /CYC4/SOMY,SOMZ
     &       /CYC5/SOMX2,SOMY2,SOMZ2
     &       /CYC6/EN
     &       /COM1/T
     &       /RES2/SNOM


      DO W=1,LW
        DO V=1,LV
          DO U=1,LU
            DO K=1,NA
              SOMX(K,U,V,W)=0
              SOMY(K,U,V,W)=0
              SOMZ(K,U,V,W)=0
              SOMX2(K,U,V,W)=0
              SOMY2(K,U,V,W)=0
              SOMZ2(K,U,V,W)=0
            END DO
          END DO
        END DO
      END DO
      IN=0
      NPAJ=0


      IF (IRAN.EQ.1) THEN          ! ISING SPINS

C
C***** MONTE CARLO LOOPS FOR ISING SPINS *****
C

        DO 102 ITIME=1,IT

          DO W=1,LW
            DO V=1,LV
              DO U=1,LU
                DO N=1,NA

                  CALL EN1_CALC(N,U,V,W)

                  ASUP=-DELTA/T-88.029
                  IF(ASUP.GT.0) GO TO 500
                  AINF=-DELTA/T+89.415
                  IF(AINF.LT.0) GO TO 10
                  EX=EXP(-DELTA/T)
                  IF(EX-RAN(I1,I2))10,500,500
C                 IF(EX-RANF())10,500,500
  500             EN=EN+DELTA
                  SX(N,U,V,W)=X
                  SY(N,U,V,W)=Y
                  SZ(N,U,V,W)=Z
                  IF (ITIME.GT.NOM) NPAJ=NPAJ+1
   10           END DO
              END DO
            END DO
          END DO
C
C * * * MEAN VALUES AND VARIANCE OF MAGNETIC MOMENTS AND ENERGY * * *
C FIRST PART OF CALCULATION: SOMMATION OVER THE SNOM=IT-NOM LAST CYCLES
C
          IF(ITIME-NOM)102,102,103
  103     IN=IN+1
          DO W=1,LW
            DO V=1,LV
              DO U=1,LU
                DO K=1,NA
                  OMX=OMX+SX(K,U,V,W)
                  OMY=OMY+SY(K,U,V,W)
                  OMZ=OMZ+SZ(K,U,V,W)
                  SOMX(K,U,V,W)=SOMX(K,U,V,W)+SX(K,U,V,W)
                  SOMY(K,U,V,W)=SOMY(K,U,V,W)+SY(K,U,V,W)
                  SOMZ(K,U,V,W)=SOMZ(K,U,V,W)+SZ(K,U,V,W)
                  SOMX2(K,U,V,W)=SOMX2(K,U,V,W)+SX(K,U,V,W)**2
                  SOMY2(K,U,V,W)=SOMY2(K,U,V,W)+SY(K,U,V,W)**2
                  SOMZ2(K,U,V,W)=SOMZ2(K,U,V,W)+SZ(K,U,V,W)**2
                END DO
              END DO
            END DO
          END DO
          ENM=ENM+EN
          EN2M=EN2M+EN*EN
          MAG2=MAG2+OMX**2+OMY**2+OMZ**2
          OMX=0
          OMY=0
          OMZ=0
  102   END DO
C
C***** END OF MONTE CARLO LOOPS FOR ISING SPINS *****
C

      ELSE IF (IRAN.EQ.2) THEN     ! XY SPINS

C
C***** MONTE CARLO LOOPS FOR XY SPINS *****
C

        DO 202 ITIME=1,IT

          DO W=1,LW
            DO V=1,LV
              DO U=1,LU
                DO N=1,NA

                  CALL EN2_CALC(N,U,V,W)

                  ASUP=-DELTA/T-88.029
                  IF(ASUP.GT.0) GO TO 502
                  AINF=-DELTA/T+89.415
                  IF(AINF.LT.0) GO TO 12
                  EX=EXP(-DELTA/T)
                  IF(EX-RAN(I1,I2))12,502,502
C                 IF(EX-RANF())12,502,502
  502             EN=EN+DELTA
                  SX(N,U,V,W)=X
                  SY(N,U,V,W)=Y
                  IF (ITIME.GT.NOM) NPAJ=NPAJ+1
   12           END DO
              END DO
            END DO
          END DO
C
C * * * MEAN VALUES AND VARIANCE OF MAGNETIC MOMENTS AND ENERGY * * *
C FIRST PART OF CALCULATION: SOMMATION OVER THE SNOM=IT-NOM LAST CYCLES
C
          IF(ITIME-NOM)202,202,203
  203     IN=IN+1
          DO W=1,LW
            DO V=1,LV
              DO U=1,LU
                DO K=1,NA
                  OMX=OMX+SX(K,U,V,W)
                  OMY=OMY+SY(K,U,V,W)
                  SOMX(K,U,V,W)=SOMX(K,U,V,W)+SX(K,U,V,W)
                  SOMY(K,U,V,W)=SOMY(K,U,V,W)+SY(K,U,V,W)
                  SOMX2(K,U,V,W)=SOMX2(K,U,V,W)+SX(K,U,V,W)**2
                  SOMY2(K,U,V,W)=SOMY2(K,U,V,W)+SY(K,U,V,W)**2
                END DO
              END DO
            END DO
          END DO
          ENM=ENM+EN
          EN2M=EN2M+EN*EN
          MAG2=MAG2+OMX**2+OMY**2
          OMX=0
          OMY=0
          OMZ=0
  202   END DO
C
C***** END OF MONTE CARLO LOOPS FOR XY SPINS *****
C

      ELSE IF (IRAN.EQ.3) THEN     ! HEISENBERG SPINS

C
C***** MONTE CARLO LOOPS FOR HEISENBERG SPINS *****
C

        DO 302 ITIME=1,IT

          DO W=1,LW
            DO V=1,LV
              DO U=1,LU
                DO N=1,NA

                  CALL EN3_CALC(N,U,V,W)

                  ASUP=-DELTA/T-88.029
                  IF(ASUP.GT.0) GO TO 503
                  AINF=-DELTA/T+89.415
                  IF(AINF.LT.0) GO TO 13
                  EX=EXP(-DELTA/T)
                  IF(EX-RAN(I1,I2))13,503,503
C                 IF(EX-RANF())13,503,503
  503             EN=EN+DELTA
                  SX(N,U,V,W)=X
                  SY(N,U,V,W)=Y
                  SZ(N,U,V,W)=Z
                  IF (ITIME.GT.NOM) NPAJ=NPAJ+1
   13           END DO
              END DO
            END DO
          END DO
C
C * * * MEAN VALUES AND VARIANCE OF MAGNETIC MOMENTS AND ENERGY * * *
C FIRST PART OF CALCULATION: SOMMATION OVER THE SNOM=IT-NOM LAST CYCLES
C
          IF(ITIME-NOM)302,302,303
  303     IN=IN+1
          DO W=1,LW
            DO V=1,LV
              DO U=1,LU
                DO K=1,NA
                  OMX=OMX+SX(K,U,V,W)
                  OMY=OMY+SY(K,U,V,W)
                  OMZ=OMZ+SZ(K,U,V,W)
                  SOMX(K,U,V,W)=SOMX(K,U,V,W)+SX(K,U,V,W)
                  SOMY(K,U,V,W)=SOMY(K,U,V,W)+SY(K,U,V,W)
                  SOMZ(K,U,V,W)=SOMZ(K,U,V,W)+SZ(K,U,V,W)
                  SOMX2(K,U,V,W)=SOMX2(K,U,V,W)+SX(K,U,V,W)**2
                  SOMY2(K,U,V,W)=SOMY2(K,U,V,W)+SY(K,U,V,W)**2
                  SOMZ2(K,U,V,W)=SOMZ2(K,U,V,W)+SZ(K,U,V,W)**2
                END DO
              END DO
            END DO
          END DO
          ENM=ENM+EN
          EN2M=EN2M+EN*EN
          MAG2=MAG2+OMX**2+OMY**2+OMZ**2
          OMX=0
          OMY=0
          OMZ=0
  302   END DO

C
C***** END OF MONTE CARLO LOOPS FOR HEISENBERG SPINS *****
C

      ELSE IF (IRAN.EQ.4) THEN     ! q-STATE POTTS PLANAR SPINS

C
C***** MONTE CARLO LOOPS FOR q-STATE PLANAR POTTS SPINS *****
C

        DO 402 ITIME=1,IT

          DO W=1,LW
            DO V=1,LV
              DO U=1,LU
                DO N=1,NA

                  CALL ENq_CALC(N,U,V,W)

                  ASUP=-DELTA/T-88.029
                  IF(ASUP.GT.0) GO TO 522
                  AINF=-DELTA/T+89.415
                  IF(AINF.LT.0) GO TO 22
                  EX=EXP(-DELTA/T)
                  IF(EX-RAN(I1,I2))22,522,522
C                 IF(EX-RANF())22,522,522
  522             EN=EN+DELTA
                  SX(N,U,V,W)=X
                  SY(N,U,V,W)=Y
                  IF (ITIME.GT.NOM) NPAJ=NPAJ+1
   22           END DO
              END DO
            END DO
          END DO
C
C * * * MEAN VALUES AND VARIANCE OF MAGNETIC MOMENTS AND ENERGY * * *
C FIRST PART OF CALCULATION: SOMMATION OVER THE SNOM=IT-NOM LAST CYCLES
C
          IF(ITIME-NOM)402,402,403
  403     IN=IN+1
          DO W=1,LW
            DO V=1,LV
              DO U=1,LU
                DO K=1,NA
                  OMX=OMX+SX(K,U,V,W)
                  OMY=OMY+SY(K,U,V,W)
                  SOMX(K,U,V,W)=SOMX(K,U,V,W)+SX(K,U,V,W)
                  SOMY(K,U,V,W)=SOMY(K,U,V,W)+SY(K,U,V,W)
                  SOMX2(K,U,V,W)=SOMX2(K,U,V,W)+SX(K,U,V,W)**2
                  SOMY2(K,U,V,W)=SOMY2(K,U,V,W)+SY(K,U,V,W)**2
                END DO
              END DO
            END DO
          END DO
          ENM=ENM+EN
          EN2M=EN2M+EN*EN
          MAG2=MAG2+OMX**2+OMY**2
          OMX=0
          OMY=0
          OMZ=0
  402   END DO

C
C***** END OF MONTE CARLO LOOPS FOR q-STATE POTTS SPINS *****
C

      END IF

      DO W=1,LW
        DO V=1,LV
          DO U=1,LU
            DO K=1,NA
              SOMX(K,U,V,W)=SOMX(K,U,V,W)/SNOM
              COMX(K,U,V,W)=SOMX(K,U,V,W)
              SOMY(K,U,V,W)=SOMY(K,U,V,W)/SNOM
              SOMZ(K,U,V,W)=SOMZ(K,U,V,W)/SNOM
              SOMX2(K,U,V,W)=SOMX2(K,U,V,W)/SNOM
              SOMY2(K,U,V,W)=SOMY2(K,U,V,W)/SNOM
              SOMZ2(K,U,V,W)=SOMZ2(K,U,V,W)/SNOM
              AX=SOMX2(K,U,V,W)-SOMX(K,U,V,W)**2
              IF(AX.LT.0) AX=0
              AY=SOMY2(K,U,V,W)-SOMY(K,U,V,W)**2
              IF(AY.LT.0) AY=0
              AZ=SOMZ2(K,U,V,W)-SOMZ(K,U,V,W)**2
              IF(AZ.LT.0) AZ=0
              SOMX2(K,U,V,W)=SQRT(AX)
              SOMY2(K,U,V,W)=SQRT(AY)
              SOMZ2(K,U,V,W)=SQRT(AZ)
            END DO
          END DO
        END DO
      END DO

      RETURN

      END


	SUBROUTINE EN_TOT(L)

C***********************************************************************
C
C  This subroutine computes the total energy of the system.
C
C  This subroutine calls the following subroutine:
C     EN3_CALC(N,U,V,W)
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'

      DIMENSION
     &   AMPS(NAMAX),AMPS2(NAMAX)
 
 
      INTEGER U,V,W
 
      REAL*8 EN,EN0,EN1
 
      COMMON
     &       /PALL/LU,LV,LW
     &       /COM2/NA
     &       /CYC8/DELTA,X,Y,Z,NFIP,NMULT
     &       /COM5/AMPS,AMPS2
     &       /CYC6/EN
     &       /CYC11/NFA
     &       /RES3/HK
 
      NMULT=2 
      DO I=1,NA
        AMPS2(I)=0
      END DO
      X=0
      Y=0
      Z=0
      EN0=0
      NFIP=0
      DO W=1,LW
        DO V=1,LV
          DO U=1,LU
            DO N=1,NA
              CALL EN3_CALC(N,U,V,W)
              EN0=EN0-DELTA
            END DO
          END DO
        END DO
      END DO
      EN=EN0/2
      EN1=EN/NFA
      IF (L.EQ.1) THEN
        WRITE(LPT,1)EN1
        WRITE(ITTO,1)EN1
      ELSE
        WRITE(LPT,2)EN1
        WRITE(ITTO,2)EN1
      END IF
      NFIP=1
      DO I=1,NA
        AMPS2(I)=AMPS(I)
      END DO
      NMULT=1

    1 FORMAT(7X,'*  Energy of the starting spin configuration = ',
     &       E12.5)
    2 FORMAT(7X,'*  Energy of the last spin configuration = ',
     &       E12.5,//)

      RETURN

      END


      SUBROUTINE CONST_FUNCT

C***********************************************************************
C
C  This subroutine calculates the constraint function Fc of the system
C  (for isotropic exchange only), according to the formula:
C  (with the convention that a vector V' represents the transposed 
C  form of a vector V)
C
C       Fc = -E/Eb  with  E  = - JijSi'Sj             (energy)
C                    and  Eb = -|Jij||Si||Sj|      (basic energy)
C
C  Further details about the constraint function are given in the
C  following paper:
C  The constaint functions: an attempt to evaluate the constraint 
C  rate inside compounds that undergo ordered magnetic frustration.
C  P. Lacorre, J. Phys. C: Solid State Phys. 20 (1987) L775-L781
C
C  This subroutine calls the following subroutine:
C     BOUND_COND
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'
      DIMENSION
     &   NAV(NAMAX,NNMAX),
     &   NVS(NAMAX),
     &   NU(NNMAX),NV(NNMAX),NW(NNMAX),
     &   AJXX(NAMAX,NNMAX),
     &   AJYY(NAMAX,NNMAX),
     &   AJZZ(NAMAX,NNMAX),
     &   COMX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   TMOM(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   FJSX(NAMAX),FJSY(NAMAX),FJSZ(NAMAX),FJSM(NAMAX)

      INTEGER U,V,W

 
      COMMON
     &       /PALL/LU,LV,LW
     &       /COM2/NA
     &       /COM8/NVS
     &       /CONDL/NU,NV,NW
     &       /TEST/NAV
     &       /CYC1/AJXX,AJYY,AJZZ
     &       /CYC4/SOMY,SOMZ
     &       /SOMM1/TMOM
     &       /SOMM2/COMX
     &       /FCO/FCONST,RC

      FCNU = 0
      FCDE = 0
      DO W=1,LW
        DO V=1,LV
          DO U=1,LU
            DO N=1,NA
              FJSX(N)=0
              FJSY(N)=0
              FJSZ(N)=0
              FJSM(N)=0
              DO K=1,NVS(N)

                CALL BOUND_COND(N,K,U,V,W)

              FJSX(N)=FJSX(N)+AJXX(N,K)*COMX(NAV(N,K),NU(K),NV(K),NW(K))
              FJSY(N)=FJSY(N)+AJYY(N,K)*SOMY(NAV(N,K),NU(K),NV(K),NW(K))
              FJSZ(N)=FJSZ(N)+AJZZ(N,K)*SOMZ(NAV(N,K),NU(K),NV(K),NW(K))
              FJSM(N)=FJSM(N)+ABS(AJXX(N,K))
     &                        *TMOM(NAV(N,K),NU(K),NV(K),NW(K))
              END DO
C
C * * *	  COMPUTATION OF THE NUMERATOR (FCNU=2*(MEAN ENERGY))   * * *
C * * *   AND DENOMINATOR (FCDE=-2*(BASIC MEAN ENERGY)) OF FC   * * *
C
              FCNU = FCNU-COMX(N,U,V,W)*FJSX(N)
     &                   -SOMY(N,U,V,W)*FJSY(N)
     &                   -SOMZ(N,U,V,W)*FJSZ(N)
              FCDE = FCDE+TMOM(N,U,V,W)*FJSM(N)
            END DO
          END DO
        END DO
      END DO
      IF (FCDE.LT.0.000001) FCDE=0.000001
      FCONST=FCNU/FCDE
      RC=50*(1+FCONST)

      RETURN

      END



      SUBROUTINE PRO_ROTA

C***********************************************************************
C
C  This subroutine computes, at the end of simulation, the projection
C  of magnetic moments onto the crystal axes, and rotates the whole
C  system in order to align a given magnetic moment along the desired
C  direction.
C  Calculation for rotation are performed within the orthonormal coord-
C  inate system, then magnetic moments are projected onto the crystal-
C  lographic reference frame.
C  The output file MYFILE.LIS stores :
C  - the components of magnetic moments inside the crystallographic
C    reference frame before rotation
C  - the components of magnetic moments inside the crystallographic
C    reference frame after rotation
C
C  The following parameters are used in the subroutine :
C                                                           
C                    TRANSFORMATION MATRICES           
C         between orthonormal coordinate system (o.c.s.)
C         and crystallographic reference frame (c.r.f.)
C  ( ref. : V.A.LIOPO_Sov. Phys. Crystallogr. 30 (1985) 687)
C
C  AA : transformation matrix (c.r.f.) ---> (o.c.s.) for magnetic moments
C  AB=AA for storage, AA being destroyed during inversion
C  AC=INV(AA) : transformation matrix (o.c.s.) ---> (c.r.f.)
C  AD : transformation matrix (c.r.f.) ---> (o.c.s.) for direction
C
C                           VECTORS                   
C  SORX,SORY,SORZ : components of the magnetic moment taken as reference
C  DORX,DORY,DORZ : new direction of the magnetic moment
C  UORX,UORY,UORZ : rotation axis ( UOR = SOR^DOR )
C  OM = angle(SOR,DOR)  [ SIOM = SIN(OM) , COOM = COS(OM) ]
C
C                       ROTATION FORMULA
C                       ----------------                   
C  ( ref.: J.BASS_Cours de Mathematiques T.I f.1 MASSON(1977) )
C                       ( p 130 - ex 36 )                    
C	                                                         
C  if S = arbitrary vector
C     S'= transformation of S by the rotation
C     U = normalised UOR
C
C         S' = (1-COOM)*(U.S)*U + COOM*S + SIOM*(U^S)       
C         -------------------------------------------       
C
C
C  This subroutine calls the following subroutine:
C     INV_MAT
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'

      DIMENSION
     &   COMX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMRX(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMRY(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMRZ(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMX2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMY2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMZ2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMRX2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMRY2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMRZ2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   ISIGX(NAMAX),ISIGY(NAMAX),ISIGZ(NAMAX)


      INTEGER U,V,W,NOR


      REAL DORX,DORY,DORZ

      REAL*8
     &       AA(3,3),AB(3,3),AC(3,3),AD(3,3),WK(3)


      COMMON 
     &       /PALL/LU,LV,LW
     &       /COM2/NA
     &       /CYC4/SOMY,SOMZ
     &       /CYC5/SOMX2,SOMY2,SOMZ2
     &       /SOMM2/COMX
     &       /PRO/SOMRX,SOMRY,SOMRZ,SOMRX2,SOMRY2,SOMRZ2
     &       /INV/AA,AB,AC


      READ(I01,*)AAX,AAY,AAZ,AAA,AAB,AAG
C
C * * * COMPUTATION OF TRANSFORMATION MATRICES AA(I,J) AND AD(I,J) * * *
C
      AAR=SQRT( 1-COSD(AAA)*COSD(AAA)
     &           -COSD(AAB)*COSD(AAB)
     &           -COSD(AAG)*COSD(AAG)
     &           +2*COSD(AAA)*COSD(AAB)*COSD(AAG) )
      AA(1,1)=1
      AA(1,2)=COSD(AAG)
      AA(1,3)=COSD(AAB)
      AA(2,2)=SIND(AAG)
      AA(2,3)=(COSD(AAB)*COSD(AAG)-COSD(AAA))/SIND(AAG)
      AA(3,3)=AAR/SIND(AAG)
      AD(1,1)=AAX
      AD(1,2)=AAY*COSD(AAG)
      AD(1,3)=AAZ*COSD(AAB)
      AD(2,2)=AAY*SIND(AAG)
      AD(2,3)=AAZ*(COSD(AAB)*COSD(AAG)-COSD(AAA))/SIND(AAG)
      AD(3,3)=AAZ*AAR/SIND(AAG)

      DO I=1,3
        DO J=1,3
          AB(I,J)=AA(I,J)
        END DO
      END DO

      CALL INV_MAT

C
C * * * NOR = INDEX OF MOMENT TAKEN AS REFERENCE (CELL (1,1,1)) * * *
C
 1119 WRITE(ITTO,620)
      READ(ITTI,*,ERR=1119)NOR
      IF ((NOR.LT.0).OR.(NOR.GT.NA)) GO TO 1119
      WRITE(IBA,*)NOR

      IF (NOR.NE.0) THEN

        SORX=COMX(NOR,1,1,1)
        SORY=SOMY(NOR,1,1,1)
        SORZ=SOMZ(NOR,1,1,1)

 1120   WRITE(ITTO,621)NOR
        READ(ITTI,*,ERR=1120)DORX,DORY,DORZ
C DORX,DORY,DORZ : NEW DIRECTION OF MOMENT NOR INSIDE THE CRYSTAL. REFERENCE

        WRITE(IBA,*)DORX,DORY,DORZ
        DORIX=DORX
        DORIY=DORY
        DORIZ=DORZ

        DORX=DORX*AD(1,1)+DORY*AD(1,2)+DORZ*AD(1,3)
        DORY=DORX*AD(2,1)+DORY*AD(2,2)+DORZ*AD(2,3)
        DORZ=DORX*AD(3,1)+DORY*AD(3,2)+DORZ*AD(3,3)
C DORX,DORY,DORZ : NEW DIRECTION OF MOMENT NOR INSIDE THE ORTHO. SYSTEM

C 
C * * * CALCULATION INSIDE THE ORTHONORMAL COORDINATE SYSTEM * * *
C
        DOR=SQRT(DORX*DORX+DORY*DORY+DORZ*DORZ)
        SORM=SQRT(SORX*SORX+SORY*SORY+SORZ*SORZ)
        SOR=DORX*SORX+DORY*SORY+DORZ*SORZ
        COOM=SOR/(DOR*SORM)
        UORX=SORY*DORZ-SORZ*DORY
        UORY=SORZ*DORX-SORX*DORZ
        UORZ=SORX*DORY-SORY*DORX
        UOR=SQRT(UORX*UORX+UORY*UORY+UORZ*UORZ)
        SIOM=UOR/(DOR*SORM)
        UORX=UORX/UOR
        UORY=UORY/UOR
        UORZ=UORZ/UOR

        DO W=1,LW
          DO V=1,LV
            DO U=1,LU
              DO K=1,NA

                PSCAL= UORX*COMX(K,U,V,W)
     &                +UORY*SOMY(K,U,V,W)
     &                +UORZ*SOMZ(K,U,V,W)
                SOMRX(K,U,V,W)= (1-COOM)*PSCAL*UORX
     &                         + COOM*COMX(K,U,V,W)
     &                         + SIOM*( UORY*SOMZ(K,U,V,W)
     &                                 -UORZ*SOMY(K,U,V,W) )
                SOMRY(K,U,V,W)= (1-COOM)*PSCAL*UORY
     &                         + COOM*SOMY(K,U,V,W)
     &                         + SIOM*( UORZ*COMX(K,U,V,W)
     &                                 -UORX*SOMZ(K,U,V,W) )
                SOMRZ(K,U,V,W)= (1-COOM)*PSCAL*UORZ
     &                         + COOM*SOMZ(K,U,V,W)
     &                         + SIOM*( UORX*SOMY(K,U,V,W)
     &                                 -UORY*COMX(K,U,V,W) )
C
C   *   SOMRX           COMPONENTS OF MAGNETIC MOMENTS   *
C   *   SOMRY   ------> INSIDE THE ORTHONORMAL SYSTEM    *
C   *   SOMRZ           AFTER ROTATION                   *
C

                SOMRX2(K,U,V,W)= (1-COOM)*PSCAL*UORX
     &                          + COOM*SOMX2(K,U,V,W)
     &                          + SIOM*( UORY*SOMZ2(K,U,V,W)
     &                                  -UORZ*SOMY2(K,U,V,W) )
                SOMRY2(K,U,V,W)= (1-COOM)*PSCAL*UORY
     &                          + COOM*SOMY2(K,U,V,W)
     &                          + SIOM*( UORZ*SOMX2(K,U,V,W)
     &                                 -UORX*SOMZ2(K,U,V,W) )
                SOMRZ2(K,U,V,W)= (1-COOM)*PSCAL*UORZ
     &                          + COOM*SOMZ2(K,U,V,W)
     &                          + SIOM*( UORX*SOMY2(K,U,V,W)
     &                                  -UORY*SOMX2(K,U,V,W) )
C
C   *   SOMRX2          VARIANCE OF THE COMPONENTS OF       *
C   *   SOMRY2  ------> MAGNETIC MOMENTS INSIDE THE         *
C   *   SOMRZ2          ORTHONORMAL SYSTEM AFTER ROTATION   *
C

              END DO
            END DO
          END DO
        END DO

C
C * * * END OF CALCULATION INSIDE THE ORTHONORMAL SYSTEM * * *
C

C
C * * * COMPUTATION INSIDE THE CRISTALLOGRAPHIC REFERENCE FRAME * * *
C

C  
C   *   PROJECTION OF MAGNETIC MOMENTS AFTER ROTATION   *
C                  (SOMRX,SOMRY,SOMRZ)
C
        DO W=1,LW
          DO V=1,LV
            DO U=1,LU
              DO K=1,NA
                SOMRX(K,U,V,W)= AC(1,1)*SOMRX(K,U,V,W)
     &                         +AC(1,2)*SOMRY(K,U,V,W)
     &                         +AC(1,3)*SOMRZ(K,U,V,W)
                SOMRY(K,U,V,W)= AC(2,1)*SOMRX(K,U,V,W)
     &                         +AC(2,2)*SOMRY(K,U,V,W)
     &                         +AC(2,3)*SOMRZ(K,U,V,W)
                SOMRZ(K,U,V,W)= AC(3,1)*SOMRX(K,U,V,W)
     &                         +AC(3,2)*SOMRY(K,U,V,W)
     &                         +AC(3,3)*SOMRZ(K,U,V,W)
                SOMRX2(K,U,V,W)= AC(1,1)*SOMRX2(K,U,V,W)
     &                          +AC(1,2)*SOMRY2(K,U,V,W)
     &                          +AC(1,3)*SOMRZ2(K,U,V,W)
                SOMRY2(K,U,V,W)= AC(2,1)*SOMRX2(K,U,V,W)
     &                          +AC(2,2)*SOMRY2(K,U,V,W)
     &                          +AC(2,3)*SOMRZ2(K,U,V,W)
                SOMRZ2(K,U,V,W)= AC(3,1)*SOMRX2(K,U,V,W)
     &                          +AC(3,2)*SOMRY2(K,U,V,W)
     &                          +AC(3,3)*SOMRZ2(K,U,V,W)
              END DO
            END DO
          END DO
        END DO

      END IF
C  
C   *   PROJECTION OF MAGNETIC MOMENTS BEFORE ROTATION   *
C                      (COMX,SOMY,SOMZ)         
C
      DO W=1,LW
        DO V=1,LV
          DO U=1,LU
            DO K=1,NA
              COMX(K,U,V,W)= AC(1,1)*COMX(K,U,V,W)
     &                      +AC(1,2)*SOMY(K,U,V,W)
     &                      +AC(1,3)*SOMZ(K,U,V,W)
              SOMY(K,U,V,W)= AC(2,1)*COMX(K,U,V,W)
     &                      +AC(2,2)*SOMY(K,U,V,W)
     &                      +AC(2,3)*SOMZ(K,U,V,W)
              SOMZ(K,U,V,W)= AC(3,1)*COMX(K,U,V,W)
     &                      +AC(3,2)*SOMY(K,U,V,W)
     &                      +AC(3,3)*SOMZ(K,U,V,W)
              SOMX2(K,U,V,W)= AC(1,1)*SOMX2(K,U,V,W)
     &                       +AC(1,2)*SOMY2(K,U,V,W)
     &                       +AC(1,3)*SOMZ2(K,U,V,W)
              SOMY2(K,U,V,W)= AC(2,1)*SOMX2(K,U,V,W)
     &                       +AC(2,2)*SOMY2(K,U,V,W)
     &                       +AC(2,3)*SOMZ2(K,U,V,W)
              SOMZ2(K,U,V,W)= AC(3,1)*SOMX2(K,U,V,W)
     &                       +AC(3,2)*SOMY2(K,U,V,W)
     &                       +AC(3,3)*SOMZ2(K,U,V,W)
            END DO
          END DO
        END DO
      END DO
C
C ***** END OF CALCULATION FOR PROJECTION AND ROTATION *****
C

C
C ***** OUTPUT RESULTS OF PROJECTION AND ROTATION *****
C
      WRITE(LPT,517)AAX,AAY,AAZ,AAA,AAB,AAG
      WRITE(ITTO,495)
      WRITE(LPT,495)
      WRITE(ITTO,618)
      WRITE(LPT,618)
      DO W=1,LW
        DO V=1,LV
          DO U=1,LU
            DO N=1,NA
              ISIGX(N)=IFIX(1000*SOMX2(N,U,V,W))
              ISIGY(N)=IFIX(1000*SOMY2(N,U,V,W))
              ISIGZ(N)=IFIX(1000*SOMZ2(N,U,V,W))
            END DO
            NPI=1
            NPF=NA
            WRITE(LPT,498)U,V,W
            WRITE(ITTO,498)U,V,W
            WRITE(LPT,499)
            WRITE(ITTO,499)

            NPASSA=0
 3412       IF((NA-NAL*NPASSA).GT.NAL) NPF=NPI+NAL-1
            NPASSA=NPASSA+1
            WRITE(ITTO,491)
            WRITE(LPT,491)
            DO N=NPI,NPF
              WRITE(ITTO,496)N
              WRITE(LPT,496)N
            END DO
            WRITE(ITTO,499)
            WRITE(LPT,499)
            WRITE(LPT,497)(COMX(N,U,V,W),N=NPI,NPF)
            WRITE(LPT,487)(ISIGX(N),N=NPI,NPF)
            WRITE(LPT,493)(SOMY(N,U,V,W),N=NPI,NPF)
            WRITE(LPT,487)(ISIGY(N),N=NPI,NPF)
            WRITE(LPT,494)(SOMZ(N,U,V,W),N=NPI,NPF)
            WRITE(LPT,487)(ISIGZ(N),N=NPI,NPF)
            WRITE(ITTO,497)(COMX(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,487)(ISIGX(N),N=NPI,NPF)
            WRITE(ITTO,493)(SOMY(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,487)(ISIGY(N),N=NPI,NPF)
            WRITE(ITTO,494)(SOMZ(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,487)(ISIGZ(N),N=NPI,NPF)
            WRITE(ITTO,495)
            WRITE(LPT,495)
            IF (NPF.GE.NPI+NAL-1) THEN
              IF ( (NPI.NE.1).OR.(NPF.NE.NA) ) THEN
              NPI=NPF+1
              NPF=NA
              GO TO 3412
              END IF
            END IF
          END DO
        END DO
      END DO

      IF(NOR.EQ.0)GO TO 611
      WRITE(ITTO,495)
      WRITE(LPT,495)
      WRITE(ITTO,638)NOR,DORIX,DORIY,DORIZ
      WRITE(LPT,638)NOR,DORIX,DORIY,DORIZ
      WRITE(ITTO,495)
      WRITE(LPT,495)
      WRITE(ITTO,628)
      WRITE(LPT,628)
      DO W=1,LW
        DO V=1,LV
          DO U=1,LU
            NPI=1
            NPF=NA
            WRITE(LPT,498)U,V,W
            WRITE(ITTO,498)U,V,W
            WRITE(LPT,499)
            WRITE(ITTO,499)

            NPASSB=0
 2412       IF((NA-NAL*NPASSB).GT.NAL) NPF=NPI+NAL-1
            NPASSB=NPASSB+1
            WRITE(ITTO,491)
            WRITE(LPT,491)
            DO N=NPI,NPF
              WRITE(ITTO,496)N
              WRITE(LPT,496)N
            END DO
            WRITE(ITTO,499)
            WRITE(LPT,499)
            WRITE(LPT,497)(SOMRX(N,U,V,W),N=NPI,NPF)
            WRITE(LPT,493)(SOMRY(N,U,V,W),N=NPI,NPF)
            WRITE(LPT,494)(SOMRZ(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,497)(SOMRX(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,493)(SOMRY(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,494)(SOMRZ(N,U,V,W),N=NPI,NPF)
            WRITE(ITTO,495)
            WRITE(LPT,495)
            IF (NPF.GE.NPI+NAL-1) THEN
              IF ( (NPI.NE.1).OR.(NPF.NE.NA) ) THEN
                NPI=NPF+1
                NPF=NA
                GO TO 2412
              END IF
            END IF
          END DO
        END DO
      END DO


  613 FORMAT(1X,3(F8.4,1X),6X,3(F8.4,1X))
  495 FORMAT(/)
  496 FORMAT('+Site',I2,'   ',$)
  497 FORMAT(1X,' Mx ',50(1X,F6.3,2X))
  493 FORMAT(1X,' My ',50(1X,F6.3,2X))
  494 FORMAT(1X,' Mz ',50(1X,F6.3,2X))
  487 FORMAT(5X,50(3X,I4,2X))
  491 FORMAT('+     ',$)
  499 FORMAT()
  498 FORMAT(1X,' CELL (',I2,',',I2,',',I2,')')
  620 FORMAT(1X,' DO YOU WANT TO REDEFINE THE ORIENTATION OF'
     &       ' MAGNETIC MOMENTS',/,'  IN THE CRYSTAL CELL ?',/,
     &       ' 0 : do not',/,
     &       ' N =< NA : ordinal number of the magnetic moment to be'
     &       ' used to calculate the new orientation')
  621 FORMAT(1X,' GIVE NEW ORIENTATION OF MAGNETIC MOMENT OF SITE ',
     &       I2,' IN THE CRYSTAL CELL :')
  517 FORMAT(1X,' CRYSTAL CELL PARAMETERS :',/,5X,' a = ',F9.4,
     &      /,5X,' b = ',F9.4,/,5X,' c = ',F9.4,/,5X,' alpha = ',F9.4,
     &      /,5X,' beta  = ',F9.4,/,5X,' gamma = ',F9.4)
  618 FORMAT(1X,' COMPONENTS OF THE MAGNETIC MOMENTS IN THE CRYSTAL'
     &       ' CELL ',/,'   *************************************'
     &       '***************',/)
  638 FORMAT(1X,' ***** MOMENTS ARE ROTATED TO ALIGN MOMENT OF SITE ',
     &       I2,' ALONG THE DIRECTION *****',/,13X,'***** ',
     &       3(F7.3,1X),'OF THE CRYSTAL CELL  *****')
  628 FORMAT(1X,' COMPONENTS OF THE MAGNETIC MOMENTS IN THE CRYSTAL'
     &       ' CELL AFTER ROTATION',/,'   ***********************'
     &       '********************************************',/)

  611 RETURN

      END


      SUBROUTINE OUT_RES

C***********************************************************************
C
C  This subroutine outputs the averaged characteristics in files
C  MYFILE.LIS and MYFILE.RES (if activated) at the end of every 
C  temperature/field loop.
C
C  The calculated averaged characteristics are the following :
C  - mean total energy (ENMA)
C  - mean total magnetization (AMMCA)
C  - mean total magnetization along the applied field (AMPARA)
C  - mean magnetization on three selected sites of cell (1,1,1)
C  - specific heat (CVA)
C  - magnetic susceptibility (SUSA)
C  - inverse susceptibility (SUAM1)
C
C  The specific heat and magnetic susceptibility are computed according
C  to the formula :
C  CVA = (<E**2>-<E>**2)/T**2    (where E represents the energy)
C  SUSA = (<M**2>-<M>**2)/T      (where M represents the magnetization)
C
C  This subroutine calls no subroutine.
C
C***********************************************************************

      INCLUDE 'MCMAG.INC'

      DIMENSION
     &   TMOM(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1)

      REAL*8 ENM,ENM2,EN2M,MAG2,EN,ENMIN

      COMMON
     &       /RES2/SNOM
     &       /RES3/HK,HDIX,HDIY,HDIZ,J1,J2,J3
     &       /RES1/LAST,ANSRF
     &       /CYC6/EN
     &       /COM7/NTA
     &       /COM1/T
     &       /FIELD/HX,HY,HZ
     &       /CYC7/ENM,EN2M,MAG2
     &       /COM6/NPAJ
     &       /CYC11/NFA,HX0,HY0,HZ0
     &       /FCO/FCONST,RC
     &       /SOMM1/TMOM,AMX,AMY,AMZ
     &       /TEST2/JCOD


      PAJ=100*NPAJ/(SNOM*NTA)

      ENM=ENM/SNOM
      ENM2=ENM*ENM
      EN2M=EN2M/SNOM
      MAG2=MAG2/SNOM
      AMMC=SQRT(AMX*AMX+AMY*AMY+AMZ*AMZ)

      ENMA=ENM/NFA
      AMPARA=HDIX*AMX+HDIY*AMY+HDIZ*AMZ
      AMMCA=AMMC/NFA
      AMPARA=AMPARA/NFA
      IF (AMMC.NE.0.) THEN
        XM=AMX/AMMC
        YM=AMY/AMMC
        ZM=AMZ/AMMC
      ELSE
        XM=0.
        YM=0.
        ZM=0.
      END IF
      SUSA=(MAG2-AMMC**2)/T
      SUSA=SUSA/NFA
      IF (SUSA.LE.0.) THEN
        SUMA1=9999.999
        GO TO 1120
      END IF
      SUAM1=1/SUSA
      IF(ABS(SUAM1).GT.9999.999)SUAM1=9999.999
 1120 CVA=(EN2M-ENM2)/(T*T)
      CVA=CVA/NFA
      IF(CVA.GT.9999.999) CVA=9999.999
      ENM=0
      EN2M=0
      MAG2=0
      AMMC=0
      IF (LAST.EQ.1) LAST=0
      IF (JCOD.NE.0) THEN
        WRITE(LPT,993)PAJ,XM,YM,ZM,HX0,HY0,HZ0
        WRITE(ITTO,993)PAJ,XM,YM,ZM,HX0,HY0,HZ0
      ELSE
        WRITE(LPT,693)PAJ,FCONST,RC,XM,YM,ZM,HX0,HY0,HZ0
        WRITE(ITTO,693)PAJ,FCONST,RC,XM,YM,ZM,HX0,HY0,HZ0
      END IF
      WRITE(ITTO,263)J1,J2,J3
      WRITE(LPT,263)J1,J2,J3
      WRITE(ITTO,264)SUSA,SUAM1,CVA,ENMA,AMMCA,
     &               TMOM(J1,1,1,1),TMOM(J2,1,1,1),TMOM(J3,1,1,1),
     &               AMPARA
      WRITE(LPT,264)SUSA,SUAM1,CVA,ENMA,AMMCA,
     &              TMOM(J1,1,1,1),TMOM(J2,1,1,1),TMOM(J3,1,1,1),
     &              AMPARA
      IF ((ANSRF.EQ.'Y').OR.(ANSRF.EQ.'y')) THEN
        WRITE(I03,265)T,HK,SUAM1,CVA,ENMA,AMMCA,
     &                TMOM(J1,1,1,1),TMOM(J2,1,1,1),TMOM(J3,1,1,1),
     &                AMPARA,FCONST
      END IF


  693 FORMAT(7X,'*  Rate of accepted jumps : ',F7.3,' %',/,
     &       7X,'*  Fc = ',F8.5,'   -   Rc = ',F7.3,' %',/,
     &       7X,'*  MTOT   along the direction ',3(1X,F6.3),/,
     &       7X,'*  M//    along the direction ',3(1X,F6.3))
  993 FORMAT(7X,'*  Rate of accepted jumps : ',F7.3,' %',/,
     &       7X,'*  MTOT   along the direction ',3(1X,F6.3),/,
     &       7X,'*  M//    along the direction ',3(1X,F6.3))
  264 FORMAT(7X,'*',F9.4,1X,F8.3,1X,F8.3,1X,F10.2,1X,F7.3,3(1X,F5.3),
     &       1X,F7.3)
  265 FORMAT(3X,F7.2,1X,F7.2,1X,F8.3,1X,F8.3,1X,F10.2,1X,F7.3,
     &       3(1X,F5.3),1X,F7.3,1X,F6.3)
  695 FORMAT(//)
  263 FORMAT(7X,'*    SUSC',4X,'1/SUS',4X,'SP. HT.',6X,'EN',5X,'MTOT',
     &       1X,3(1X,'M(',I2,')'),4X,'M//')

      RETURN

      END


      SUBROUTINE OUT_CONF

C***********************************************************************
C
C  This subroutine output the configuration of magnetic moments at
C  current temperature and field.
C
C  This subroutine calls no subroutine.
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'

      DIMENSION
     &   AMPS(NAMAX),
     &   SOMX(NAMAX,NUMAX,NVMAX,NWMAX),
     &   COMX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   TMOM(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SOMX2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMY2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SOMZ2(NAMAX,NUMAX,NVMAX,NWMAX),
     &   ISIGX(NAMAX),ISIGY(NAMAX),ISIGZ(NAMAX),
     &   ISIGM(NAMAX)



      INTEGER U,V,W


      COMMON 
     &       /PALL/LU,LV,LW
     &       /COM2/NA
     &       /CYC9/SOMX
     &       /CYC4/SOMY,SOMZ
     &       /CYC5/SOMX2,SOMY2,SOMZ2
     &       /COM5/AMPS
     &       /SOMM1/TMOM,AMX,AMY,AMZ
     &       /SOMM2/COMX
     &       /RES2/SNOM
     &       /RES1/LAST,ANSRF,ANSNEV
     &       /WRIT/IWMX,IWMXY


      AMX=0
      AMY=0
      AMZ=0

      IF ( ((ANSNEV.EQ.'A').OR.(ANSNEV.EQ.'a')).OR.
     &   ( ((ANSNEV.EQ.'E').OR.(ANSNEV.EQ.'e')).AND.
     &   (LAST.EQ.1) ) ) THEN
        WRITE(LPT,477)
      END IF

      DO W=1,LW
        DO V=1,LV
          DO U=1,LU

            DO K=1,NA
              TMOM(K,U,V,W)=SQRT( SOMX(K,U,V,W)**2
     &                           +SOMY(K,U,V,W)**2
     &                           +SOMZ(K,U,V,W)**2 )
            END DO  
            NPI=1
            NPF=NA
            IF ( ((ANSNEV.EQ.'A').OR.(ANSNEV.EQ.'a')).OR.
     &         ( ((ANSNEV.EQ.'E').OR.(ANSNEV.EQ.'e')).AND.
     &           (LAST.EQ.1) ) ) THEN
              WRITE(ITTO,498)U,V,W
              WRITE(LPT,498)U,V,W
              WRITE(LPT,499)
              WRITE(ITTO,499)

              NPASS=0
 1412         IF((NA-NAL*NPASS).GT.NAL) NPF=NPI+NAL-1
              NPASS=NPASS+1
              WRITE(ITTO,491)
              WRITE(LPT,491)
              DO N=NPI,NPF
                WRITE(ITTO,496)N
                WRITE(LPT,496)N
              END DO
              DO K=NPI,NPF
                ISIGX(K)=IFIX(1000*SOMX2(K,U,V,W))
                ISIGY(K)=IFIX(1000*SOMY2(K,U,V,W))
                ISIGZ(K)=IFIX(1000*SOMZ2(K,U,V,W))
              END DO

              WRITE(LPT,497)(SOMX(N,U,V,W),N=NPI,NPF)
              WRITE(LPT,487)(ISIGX(N),N=NPI,NPF)
              WRITE(ITTO,497)(SOMX(N,U,V,W),N=NPI,NPF)
              WRITE(ITTO,487)(ISIGX(N),N=NPI,NPF)
              IF(IWMX.EQ.1) GO TO 998
              WRITE(LPT,493)(SOMY(N,U,V,W),N=NPI,NPF)
              WRITE(LPT,487)(ISIGY(N),N=NPI,NPF)
              WRITE(ITTO,493)(SOMY(N,U,V,W),N=NPI,NPF)
              WRITE(ITTO,487)(ISIGY(N),N=NPI,NPF)
              IF(IWMXY.EQ.1) GO TO 998
              WRITE(LPT,494)(SOMZ(N,U,V,W),N=NPI,NPF)
              WRITE(LPT,487)(ISIGZ(N),N=NPI,NPF)
              WRITE(ITTO,494)(SOMZ(N,U,V,W),N=NPI,NPF)
              WRITE(ITTO,487)(ISIGZ(N),N=NPI,NPF)

  998         DO K=NPI,NPF
                IF (TMOM(K,U,V,W).GT.0.00001) THEN
                  SOMX(K,U,V,W)=( ABS(SOMX(K,U,V,W))*SOMX2(K,U,V,W)
     &                           +ABS(SOMY(K,U,V,W))*SOMY2(K,U,V,W)
     &                           +ABS(SOMZ(K,U,V,W))*SOMZ2(K,U,V,W) )
     &                          /TMOM(K,U,V,W)
                ELSE
                  SOMX(K,U,V,W)=AMPS(K)
                END IF
                ISIGM(K)=IFIX(1000*SOMX(K,U,V,W))
              END DO

              IF(IWMX.EQ.1) GO TO 999
              WRITE(LPT,486)(TMOM(N,U,V,W),N=NPI,NPF)
              WRITE(LPT,487)(ISIGM(N),N=NPI,NPF)
              WRITE(ITTO,486)(TMOM(N,U,V,W),N=NPI,NPF)
              WRITE(ITTO,487)(ISIGM(N),N=NPI,NPF)

  999 	      WRITE(ITTO,495)
              WRITE(LPT,495)
            END IF
            DO L=NPI,NPF
              AMX=AMX+COMX(L,U,V,W)
              AMY=AMY+SOMY(L,U,V,W)
              AMZ=AMZ+SOMZ(L,U,V,W)
            END DO
            IF (NPF.GE.NPI+NAL-1) THEN
              IF ((NPI.NE.1).OR.(NPF.NE.NA)) THEN
                NPI=NPF+1
                NPF=NA
                GO TO 1412
              END IF
            END IF

          END DO
        END DO
      END DO

  477 FORMAT(/,19X,' Mean Configuration of Magnetic Moments',/,
     &       19X,'         ---------------------')
  498 FORMAT(1X,' CELL (',I2,',',I2,',',I2,')')
  496 FORMAT('+Site',I2,'   ',$)
  497 FORMAT(1X,' Mx ',50(1X,F6.3,2X))
  493 FORMAT(1X,' My ',50(1X,F6.3,2X))
  494 FORMAT(1X,' Mz ',50(1X,F6.3,2X))
  486 FORMAT(1X,'Mtot',50(1X,F6.3,2X))
  487 FORMAT(5X,50(3X,I4,2X))
  491 FORMAT('+     ',$)
  499 FORMAT()
  495 FORMAT(/)

      RETURN

      END


      SUBROUTINE BOUND_COND(N,K,U,V,W)

C***********************************************************************
C
C  This subroutine implements the boundary conditions for spins at
C  the edges of the sample. Three kinds of boundary conditions are
C  available:
C
C  - free edges:
C    ----------
C    spins at any edge of the sample are considered as free of nei-
C    ghbours outside of the sample. The sample is then representa-
C    tive of a real small crystallite delimited by surfaces
C
C  - periodic boundary conditions (p.b.c.):
C    ----------------------------
C    spins at one edge of the sample are connected to spins at the
C    opposite edge of the sample. This condition imposes a 3D perio-
C    dicity to the system. Then the system may be thought of as an 
C    infinite size crystal with an elementary cell equal to the
C    sample dimension.
C
C  - mixed boundary conditions:
C    -------------------------
C    this new kind of boundary conditions has been introduced to
C    simulate helimagnets with a propagation axis of the helix.
C    It corresponds to free edges along one direction (axis W) and
C    p.b.c along the other two directions.
C
C     This subroutine calls no subroutine.
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'

      DIMENSION NU(NNMAX),NV(NNMAX),NW(NNMAX),
     &          NVS(NAMAX),M(NAMAX,NNMAX,3)

      INTEGER U,V,W

      COMMON
     &       /PALL/LU,LV,LW
     &       /COM4/M
     &       /CONDL/NU,NV,NW
     &       /COND2/IANB


      NU(K)=U+M(N,K,1)
      NV(K)=V+M(N,K,2)
      NW(K)=W+M(N,K,3)
      IF(IANB) 97,199,99
C
C   *	MIXED BOUNDARY CONDITIONS   *
C
   97   IF(NU(K)) 900,900,910
  900     NU(K)=NU(K)+LU
          GO TO 930
  910     IF(LU-NU(K)) 920,930,930
  920       NU(K)=NU(K)-LU
  930       IF(NV(K)) 940,940,950
  940         NV(K)=NV(K)+LV
              GO TO 970
  950         IF(LV-NV(K)) 960,970,970
  960           NV(K)=NV(K)-LV
  970           IF(NW(K)) 980,980,990
  980             NW(K)=NWMAX+1
                  GO TO 911
  990             IF(LW-NW(K)) 901,911,911
  901               NW(K)=NWMAX+1
  911               GO TO 98
C
C   *	PERIODIC BOUNDARY CONDITIONS   *
C
   99	IF(NU(K)) 100,100,110
  100     NU(K)=NU(K)+LU
          GO TO 130
  110     IF(LU-NU(K)) 120,130,130
  120       NU(K)=NU(K)-LU
  130       IF(NV(K)) 140,140,150
  140         NV(K)=NV(K)+LV
              GO TO 170
  150         IF(LV-NV(K)) 160,170,170
  160           NV(K)=NV(K)-LV
  170           IF(NW(K)) 180,180,190
  180             NW(K)=NW(K)+LW
                  GO TO 210
  190             IF(LW-NW(K)) 200,210,210
  200               NW(K)=NW(K)-LW
  210               GO TO 98
C
C   *	FREE EDGES   *
C
  199   IF(NU(K)) 300,300,310
  300     NU(K)=NUMAX+1
          GO TO 330
  310     IF(LU-NU(K)) 320,330,330
  320       NU(K)=NUMAX+1
  330       IF(NV(K)) 340,340,350
  340         NV(K)=NVMAX+1
              GO TO 370
  350         IF(LV-NV(K)) 360,370,370
  360           NV(K)=NVMAX+1
  370           IF(NW(K)) 380,380,390
  380 	          NW(K)=NWMAX+1
                  GO TO 98
  390             IF(LW-NW(K)) 400,98,98
  400               NW(K)=NWMAX+1
   98               CONTINUE

      RETURN

      END


      SUBROUTINE INV_MAT

C***********************************************************************
C
C  This subroutine computes the inverse matrix of a NPxNP matrix.
C
C  This subroutine calls no subroutine.
C
C***********************************************************************

C     SUBROUTINE FROM C. DE POLIGNAC (ILL-GRENOBLE)
C     .. Parameters ..
      PARAMETER (NP=3)
      PARAMETER (NMAX=50)
C     ..
      CHARACTER DUMMY*3
C     ..
      REAL*8 BIG,DUM,PIVINV
      REAL*8 AA(NP,NP),AB(NP,NP),AC(NP,NP),U(NP,NP)

      INTEGER INDXC(NMAX),INDXR(NMAX),IPIV(NMAX)
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     ..



      COMMON /INV/AA,AB,AC
C     ..
C Invert Matrix

      DO 11 J = 1,NP
          IPIV(J) = 0
   11 CONTINUE
      DO 22 I = 1,NP
          BIG = 0.D0
          DO 13 J = 1,NP
              IF (IPIV(J).NE.1) THEN
                  DO 12 K = 1,NP
                      IF (IPIV(K).EQ.0) THEN
                          IF (ABS(AA(J,K)).GE.BIG) THEN
                              BIG = ABS(AA(J,K))
                              IROW = J
                              ICOL = K
                          END IF

                      ELSE IF (IPIV(K).GT.1) THEN
                          PAUSE 'Singular matrix'

                      END IF

   12             CONTINUE
              END IF

   13     CONTINUE
          IPIV(ICOL) = IPIV(ICOL) + 1
          IF (IROW.NE.ICOL) THEN
              DO 14 L = 1,NP
                  DUM = AA(IROW,L)
                  AA(IROW,L) = AA(ICOL,L)
                  AA(ICOL,L) = DUM
   14         CONTINUE
          END IF

          INDXR(I) = IROW
          INDXC(I) = ICOL
          IF (AA(ICOL,ICOL).EQ.0.D0) PAUSE 'Singular matrix.'
          PIVINV = 1.D0/AA(ICOL,ICOL)
          AA(ICOL,ICOL) = 1.D0
          DO 16 L = 1,NP
              AA(ICOL,L) = AA(ICOL,L)*PIVINV
   16     CONTINUE
          DO 21 LL = 1,NP
              IF (LL.NE.ICOL) THEN
                  DUM = AA(LL,ICOL)
                  AA(LL,ICOL) = 0.D0
                  DO 18 L = 1,NP
                      AA(LL,L) = AA(LL,L) - AA(ICOL,L)*DUM
   18             CONTINUE
              END IF

   21     CONTINUE
   22 CONTINUE
      DO 24 L = NP,1,-1
          IF (INDXR(L).NE.INDXC(L)) THEN
              DO 23 K = 1,NP
                  DUM = AA(K,INDXR(L))
                  AA(K,INDXR(L)) = AA(K,INDXC(L))
                  AA(K,INDXC(L)) = DUM
   23         CONTINUE
          END IF

   24 CONTINUE

      DO I=1,NP
        DO J=1,NP
          AC(I,J)=AA(I,J)
        END DO
      END DO

      RETURN

      END



C***********************************************************************
C
C       General description of subroutines ENn_CALC(N,U,V,W)
C
C  The subroutines ENn_CALC(N,U,V,W) calculate the energy difference 
C  between the new randomly generated configuration (NEW) and the old 
C  configuration (OLD), expressed as DELTA = E(OLD) - E(NEW) 
C  according to the energy formula :
C  (with the convention that a vector V' represents the transposed 
C  form of a vector V)
C 
C  E = - 0.5*Si'[Jij]Sj + ((Di'Si)**2)/|Di| - H'Si
C                                              AJXX AJXY AJXZ
C  [Jij] is the exchange tensor with elements  AJYX AJYY AJYZ
C                                              AJZX AJZY AJZZ
C
C  The exchange term Si'[Jij]Sj can be written Si'Jsi where
C  Jsi = [Jij]Sj is the vector (AJSX,AJSY,AJSZ).
C  The calculation of DELTA is made in two steps:
C      - first is calculated the term Jsi, which corresponds to the
C        microscopic molecular field at site i, and that is obviously
C        the same for the two configurations of spin i
C      _ then DELTA difference is calculated according to the above
C        formula.
C
C  The four subroutines ENn_CALC(N,U,V,W) concern different spin
C  models, namely:
C      the Ising model                       with  EN1_CALC(N,U,V,W)
C      the XY model                          with  EN2_CALC(N,U,V,W)
C      the Heisenberg model                  with  EN3_CALC(N,U,V,W)
C      the q-state Potts planar (XY) model   with  ENq_CALC(N,U,V,W)
C  Only one of them is activated at every run of the program.
C
C  These subroutines call the following subroutines:
C     BOUND_COND
C     RANnD(RADIUS,X1,Y1,,)
C
C***********************************************************************


      SUBROUTINE EN1_CALC(N,U,V,W)

C***********************************************************************
C
C  See the general description of subroutines ENn_CALC(N,U,V,W)
C  just above.
C  EN1_CALC(N,U,V,W) deals with Ising spins.
C
C***********************************************************************

      INCLUDE 'MCMAG.INC'
      DIMENSION
     &   NU(NNMAX),NV(NNMAX),NW(NNMAX),
     &   NVS(NAMAX),M(NAMAX,NNMAX,3),
     &   NAV(NAMAX,NNMAX),
     &   AMPS(NAMAX),AMPS2(NAMAX),
     &   SOMX(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   AJSX(NAMAX),AJSY(NAMAX),AJSZ(NAMAX),
     &   AJJX(NAMAX,NNMAX),
     &   AJJY(NAMAX,NNMAX),
     &   AJJZ(NAMAX,NNMAX),
     &   AJXX(NAMAX,NNMAX),
     &   AJYY(NAMAX,NNMAX),
     &   AJZZ(NAMAX,NNMAX),
     &   AJXY(NAMAX,NNMAX),
     &   AJXZ(NAMAX,NNMAX),
     &   AJYX(NAMAX,NNMAX),
     &   AJYZ(NAMAX,NNMAX),
     &   AJZX(NAMAX,NNMAX),
     &   AJZY(NAMAX,NNMAX),
     &   IUM(NMOMAX),IVM(NMOMAX),IWM(NMOMAX),
     &   D(NAMAX),DX(NAMAX),DY(NAMAX),DZ(NAMAX),
     &   NM(NMOMAX),NUMB(NMOMAX),AJMOD(NMOMAX)


      INTEGER U,V,W

      REAL*8 EN


      COMMON
     &       /COM8/NVS
     &       /CONDL/NU,NV,NW
     &       /TEST/NAV,AJJX,AJJY,AJJZ,AJXY,AJXZ,AJYZ,
     &             AJYX,AJZX,AJZY
     &       /READ4/D,DX,DY,DZ
     &       /CYC1/AJXX,AJYY,AJZZ
     &       /CYC8/DELTA,X,Y,Z,NFIP,NMULT
     &       /CYC2/AJMOD,IUM,IVM,IWM,NM,NUMB,IM
     &       /CYC9/SOMX,AJSX,AJSY,AJSZ
     &       /CYC3/SX,SY,SZ
     &       /COM5/AMPS,AMPS2
     &       /FIELD/HX,HY,HZ
     &       /MOD1/ANSMOD


      AJSX(N)=0
      AJSY(N)=0
      AJSZ(N)=0
      DO K=1,NVS(N)
        AJXX(N,K)=AJJX(N,K)
        AJYY(N,K)=AJJY(N,K)
        AJZZ(N,K)=AJJZ(N,K)

        IF ((ANSMOD.EQ.'Y').OR.(ANSMOD.EQ.'y')) THEN

C
C*****    LOCAL MODIFICATION OF COUPLING (IF ANY)    ******
C

          DO I=1,IM
            IF ( (W.EQ.IWM(I)).AND.(V.EQ.IVM(I)).AND.(U.EQ.IUM(I)).
     &          AND.(N.EQ.NM(I)).AND.(K.EQ.NUMB(I)) ) THEN
              AJXX(N,K)=AJMOD(I)
              AJYY(N,K)=AJMOD(I)
              AJZZ(N,K)=AJMOD(I)
            END IF
          END DO

C
C*****    END OF MODIFICATION    *****
C

        END IF

        CALL BOUND_COND(N,K,U,V,W)

C
C*****    COMPUTE DELTA    *****
C
        IF ( (NAV(N,K).NE.N).OR.(NU(K).NE.U).OR.(NV(K).NE.V).
     &       OR.(NW(K).NE.W) ) THEN
          AJSX(N)=AJSX(N)+AJXX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJXY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJXZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
          AJSY(N)=AJSY(N)+AJYY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJYX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJYZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
          AJSZ(N)=AJSZ(N)+AJZZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJZX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJZY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
        END IF
      END DO

      IF (NFIP.NE.0) THEN 
        CALL RAN1D (AMPS2(N),X,Y,Z,N)
      END IF

      DELTA = (SX(N,U,V,W)-X)*(AJSX(N)+(NMULT*HX))
     &       +(SY(N,U,V,W)-Y)*(AJSY(N)+(NMULT*HY))
     &       +(SZ(N,U,V,W)-Z)*(AJSZ(N)+(NMULT*HZ))
     &       -NMULT*D(N) * (
     &                       ( DX(N)*SX(N,U,V,W)
     &                        +DY(N)*SY(N,U,V,W)
     &                        +DZ(N)*SZ(N,U,V,W) )**2
     &                      -( DX(N)*X + DY(N)*Y + DZ(N)*Z )**2
     &                                                   )

      RETURN

      END



      SUBROUTINE EN2_CALC(N,U,V,W)

C***********************************************************************
C
C  See the general description of subroutines EN*_CALC(N,U,V,W)
C  just before the subroutine EN1_CALC(N,U,V,W).
C  EN2_CALC(N,U,V,W) concerns XY spins.
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'

      DIMENSION
     &   NU(NNMAX),NV(NNMAX),NW(NNMAX),
     &   NVS(NAMAX),M(NAMAX,NNMAX,3),
     &   NAV(NAMAX,NNMAX),
     &   AMPS(NAMAX),AMPS2(NAMAX),
     &   SOMX(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   AJSX(NAMAX),AJSY(NAMAX),AJSZ(NAMAX),
     &   AJJX(NAMAX,NNMAX),
     &   AJJY(NAMAX,NNMAX),
     &   AJJZ(NAMAX,NNMAX),
     &   AJXX(NAMAX,NNMAX),
     &   AJYY(NAMAX,NNMAX),
     &   AJZZ(NAMAX,NNMAX),
     &   AJXY(NAMAX,NNMAX),
     &   AJXZ(NAMAX,NNMAX),
     &   AJYX(NAMAX,NNMAX),
     &   AJYZ(NAMAX,NNMAX),
     &   AJZX(NAMAX,NNMAX),
     &   AJZY(NAMAX,NNMAX),
     &   IUM(NMOMAX),IVM(NMOMAX),IWM(NMOMAX),
     &   D(NAMAX),DX(NAMAX),DY(NAMAX),DZ(NAMAX),
     &   NM(NMOMAX),NUMB(NMOMAX),AJMOD(NMOMAX)


      INTEGER U,V,W


      REAL*8 EN


      COMMON
     &       /COM8/NVS
     &       /CONDL/NU,NV,NW
     &       /TEST/NAV,AJJX,AJJY,AJJZ,AJXY,AJXZ,AJYZ,
     &             AJYX,AJZX,AJZY
     &       /READ4/D,DX,DY,DZ
     &       /CYC1/AJXX,AJYY,AJZZ
     &       /CYC8/DELTA,X,Y,Z,NFIP,NMULT
     &       /CYC2/AJMOD,IUM,IVM,IWM,NM,NUMB,IM
     &       /CYC9/SOMX,AJSX,AJSY,AJSZ
     &       /CYC3/SX,SY,SZ
     &       /COM5/AMPS,AMPS2
     &       /FIELD/HX,HY,HZ
     &       /MOD1/ANSMOD


      AJSX(N)=0
      AJSY(N)=0
      AJSZ(N)=0

      DO K=1,NVS(N)
        AJXX(N,K)=AJJX(N,K)
        AJYY(N,K)=AJJY(N,K)
        AJZZ(N,K)=AJJZ(N,K)

C
C*****    LOCAL MODIFICATION OF COUPLING (IF ANY)    ******
C

        IF ((ANSMOD.EQ.'Y').OR.(ANSMOD.EQ.'y')) THEN
          DO I=1,IM
            IF ( (W.EQ.IWM(I)).AND.(V.EQ.IVM(I)).AND.(U.EQ.IUM(I)).
     &          AND.(N.EQ.NM(I)).AND.(K.EQ.NUMB(I)) ) THEN
              AJXX(N,K)=AJMOD(I)
              AJYY(N,K)=AJMOD(I)
              AJZZ(N,K)=AJMOD(I)
            END IF
          END DO

C
C*****    END OF MODIFICATION    *****
C

        END IF

        CALL BOUND_COND(N,K,U,V,W)

C
C*****    COMPUTE DELTA    *****
C
        IF ( (NAV(N,K).NE.N).OR.(NU(K).NE.U).OR.(NV(K).NE.V).
     &      OR.(NW(K).NE.W) ) THEN
          AJSX(N)=AJSX(N)+AJXX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJXY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJXZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
          AJSY(N)=AJSY(N)+AJYY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJYX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJYZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
          AJSZ(N)=AJSZ(N)+AJZZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJZX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJZY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
        END IF
      END DO

      IF (NFIP.NE.0) THEN 
        CALL RAN2D (AMPS2(N),X,Y)
      END IF

      DELTA = (SX(N,U,V,W)-X)*(AJSX(N)+(NMULT*HX))
     &       +(SY(N,U,V,W)-Y)*(AJSY(N)+(NMULT*HY))
     &       +(SZ(N,U,V,W)-Z)*(AJSZ(N)+(NMULT*HZ))
     &       -NMULT*D(N) * (
     &                       ( DX(N)*SX(N,U,V,W)
     &                        +DY(N)*SY(N,U,V,W)
     &                        +DZ(N)*SZ(N,U,V,W) )**2
     &                      -( DX(N)*X + DY(N)*Y + DZ(N)*Z )**2
     &                                                   )

      RETURN

      END



      SUBROUTINE EN3_CALC(N,U,V,W)

C***********************************************************************
C
C  See the general description of subroutines EN*_CALC(N,U,V,W)
C  just before the subroutine EN1_CALC(N,U,V,W).
C  EN3_CALC(N,U,V,W) concerns Heisenberg spins.
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'
      DIMENSION
     &   NU(NNMAX),NV(NNMAX),NW(NNMAX),
     &   NVS(NAMAX),M(NAMAX,NNMAX,3),
     &   NAV(NAMAX,NNMAX),
     &   AMPS(NAMAX),AMPS2(NAMAX),
     &   SOMX(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   AJSX(NAMAX),AJSY(NAMAX),AJSZ(NAMAX),
     &   AJJX(NAMAX,NNMAX),
     &   AJJY(NAMAX,NNMAX),
     &   AJJZ(NAMAX,NNMAX),
     &   AJXX(NAMAX,NNMAX),
     &   AJYY(NAMAX,NNMAX),
     &   AJZZ(NAMAX,NNMAX),
     &   AJXY(NAMAX,NNMAX),
     &   AJXZ(NAMAX,NNMAX),
     &   AJYX(NAMAX,NNMAX),
     &   AJYZ(NAMAX,NNMAX),
     &   AJZX(NAMAX,NNMAX),
     &   AJZY(NAMAX,NNMAX),
     &   IUM(NMOMAX),IVM(NMOMAX),IWM(NMOMAX),
     &   D(NAMAX),DX(NAMAX),DY(NAMAX),DZ(NAMAX),
     &   NM(NMOMAX),NUMB(NMOMAX),AJMOD(NMOMAX)


      INTEGER U,V,W


      REAL*8 EN


      COMMON
     &       /COM8/NVS
     &       /CONDL/NU,NV,NW
     &       /TEST/NAV,AJJX,AJJY,AJJZ,AJXY,AJXZ,AJYZ,
     &             AJYX,AJZX,AJZY
     &       /READ4/D,DX,DY,DZ
     &       /CYC1/AJXX,AJYY,AJZZ
     &       /CYC8/DELTA,X,Y,Z,NFIP,NMULT
     &       /CYC2/AJMOD,IUM,IVM,IWM,NM,NUMB,IM
     &       /CYC9/SOMX,AJSX,AJSY,AJSZ
     &       /CYC3/SX,SY,SZ
     &       /COM5/AMPS,AMPS2
     &       /FIELD/HX,HY,HZ
     &       /MOD1/ANSMOD


      AJSX(N)=0
      AJSY(N)=0
      AJSZ(N)=0

      DO K=1,NVS(N)
        AJXX(N,K)=AJJX(N,K)
        AJYY(N,K)=AJJY(N,K)
        AJZZ(N,K)=AJJZ(N,K)

C
C*****    LOCAL MODIFICATION OF COUPLING (IF ANY)    ******
C

        IF ((ANSMOD.EQ.'Y').OR.(ANSMOD.EQ.'y')) THEN
          DO I=1,IM
            IF ( (W.EQ.IWM(I)).AND.(V.EQ.IVM(I)).AND.(U.EQ.IUM(I)).
     &          AND.(N.EQ.NM(I)).AND.(K.EQ.NUMB(I)) ) THEN
              AJXX(N,K)=AJMOD(I)
              AJYY(N,K)=AJMOD(I)
              AJZZ(N,K)=AJMOD(I)
            END IF
          END DO

C
C*****    END OF MODIFICATION    *****
C

        END IF

        CALL BOUND_COND(N,K,U,V,W)

C
C*****    COMPUTE DELTA    *****
C
        IF ( (NAV(N,K).NE.N).OR.(NU(K).NE.U).OR.(NV(K).NE.V).
     &      OR.(NW(K).NE.W) ) THEN
          AJSX(N)=AJSX(N)+AJXX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJXY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJXZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
          AJSY(N)=AJSY(N)+AJYY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJYX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJYZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
          AJSZ(N)=AJSZ(N)+AJZZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJZX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJZY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
        END IF
      END DO

      IF (NFIP.NE.0) THEN 
        CALL RAN3D (AMPS2(N),X,Y,Z)
      END IF

      DELTA = (SX(N,U,V,W)-X)*(AJSX(N)+(NMULT*HX))
     &       +(SY(N,U,V,W)-Y)*(AJSY(N)+(NMULT*HY))
     &       +(SZ(N,U,V,W)-Z)*(AJSZ(N)+(NMULT*HZ))
     &       -NMULT*D(N) * (
     &                       ( DX(N)*SX(N,U,V,W)
     &                        +DY(N)*SY(N,U,V,W)
     &                        +DZ(N)*SZ(N,U,V,W) )**2
     &                      -( DX(N)*X + DY(N)*Y + DZ(N)*Z )**2
     &                                                   )

      RETURN

      END


      SUBROUTINE ENq_CALC(N,U,V,W)

C***********************************************************************
C
C  See the general description of subroutines EN*_CALC(N,U,V,W)
C  just before the subroutine EN1_CALC(N,U,V,W).
C  ENq_CALC(N,U,V,W) concerns q-state Potts planar (XY) spins.
C
C***********************************************************************


      INCLUDE 'MCMAG.INC'
      DIMENSION
     &   NU(NNMAX),NV(NNMAX),NW(NNMAX),
     &   NVS(NAMAX),M(NAMAX,NNMAX,3),
     &   NAV(NAMAX,NNMAX),
     &   AMPS(NAMAX),AMPS2(NAMAX),
     &   SOMX(NAMAX,NUMAX,NVMAX,NWMAX),
     &   SX(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SY(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   SZ(NAMAX,NUMAX+1,NVMAX+1,NWMAX+1),
     &   AJSX(NAMAX),AJSY(NAMAX),AJSZ(NAMAX),
     &   AJJX(NAMAX,NNMAX),
     &   AJJY(NAMAX,NNMAX),
     &   AJJZ(NAMAX,NNMAX),
     &   AJXX(NAMAX,NNMAX),
     &   AJYY(NAMAX,NNMAX),
     &   AJZZ(NAMAX,NNMAX),
     &   AJXY(NAMAX,NNMAX),
     &   AJXZ(NAMAX,NNMAX),
     &   AJYX(NAMAX,NNMAX),
     &   AJYZ(NAMAX,NNMAX),
     &   AJZX(NAMAX,NNMAX),
     &   AJZY(NAMAX,NNMAX),
     &   IUM(NMOMAX),IVM(NMOMAX),IWM(NMOMAX),
     &   D(NAMAX),DX(NAMAX),DY(NAMAX),DZ(NAMAX),
     &   NM(NMOMAX),NUMB(NMOMAX),AJMOD(NMOMAX)


      INTEGER U,V,W


      REAL*8 EN


      COMMON
     &       /COM8/NVS
     &       /CONDL/NU,NV,NW
     &       /TEST/NAV,AJJX,AJJY,AJJZ,AJXY,AJXZ,AJYZ,
     &             AJYX,AJZX,AJZY
     &       /READ4/D,DX,DY,DZ
     &       /CYC1/AJXX,AJYY,AJZZ
     &       /CYC8/DELTA,X,Y,Z,NFIP,NMULT
     &       /CYC2/AJMOD,IUM,IVM,IWM,NM,NUMB,IM
     &       /CYC9/SOMX,AJSX,AJSY,AJSZ
     &       /CYC3/SX,SY,SZ
     &       /COM5/AMPS,AMPS2
     &       /FIELD/HX,HY,HZ
     &       /MOD1/ANSMOD


      AJSX(N)=0
      AJSY(N)=0
      AJSZ(N)=0

      DO K=1,NVS(N)
        AJXX(N,K)=AJJX(N,K)
        AJYY(N,K)=AJJY(N,K)
        AJZZ(N,K)=AJJZ(N,K)

C
C*****    LOCAL MODIFICATION OF COUPLING (IF ANY)    ******
C

        IF ((ANSMOD.EQ.'Y').OR.(ANSMOD.EQ.'y')) THEN
          DO I=1,IM
            IF ( (W.EQ.IWM(I)).AND.(V.EQ.IVM(I)).AND.(U.EQ.IUM(I)).
     &          AND.(N.EQ.NM(I)).AND.(K.EQ.NUMB(I)) ) THEN
              AJXX(N,K)=AJMOD(I)
              AJYY(N,K)=AJMOD(I)
              AJZZ(N,K)=AJMOD(I)
            END IF
          END DO

C
C*****    END OF MODIFICATION    *****
C

        END IF

        CALL BOUND_COND(N,K,U,V,W)

C
C*****    COMPUTE DELTA    *****
C
        IF ( (NAV(N,K).NE.N).OR.(NU(K).NE.U).OR.(NV(K).NE.V).
     &      OR.(NW(K).NE.W) ) THEN
          AJSX(N)=AJSX(N)+AJXX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJXY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJXZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
          AJSY(N)=AJSY(N)+AJYY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJYX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJYZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
          AJSZ(N)=AJSZ(N)+AJZZ(N,K)*SZ(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJZX(N,K)*SX(NAV(N,K),NU(K),NV(K),NW(K))
     &                   +AJZY(N,K)*SY(NAV(N,K),NU(K),NV(K),NW(K))
        END IF

      END DO

      IF (NFIP.NE.0) THEN 
        CALL RANqD (AMPS2(N),X,Y)
      END IF

      DELTA = (SX(N,U,V,W)-X)*(AJSX(N)+(NMULT*HX))
     &       +(SY(N,U,V,W)-Y)*(AJSY(N)+(NMULT*HY))
     &       +(SZ(N,U,V,W)-Z)*(AJSZ(N)+(NMULT*HZ))
     &       -NMULT*D(N) * (
     &                       ( DX(N)*SX(N,U,V,W)
     &                        +DY(N)*SY(N,U,V,W)
     &                        +DZ(N)*SZ(N,U,V,W) )**2
     &                      -( DX(N)*X + DY(N)*Y + DZ(N)*Z )**2
     &                                                   )

      RETURN

      END


C***********************************************************************
C
C  The subroutines RANnD(RADIUS,X1,Y1,,) generate a pseudo-random
C  spin configuration that is a random vector suitable for the selected
C  spin model:
C      RAN1D(RADIUS,X1)         for    Ising spins 
C      RAN2D(RADIUS,X1,Y1)      for    XY spins
C      RAN3D(RADIUS,X1,Y1,Z1)   for    Heisenberg spins
C      RANqD(RADIUS,X1,Y1)      for    q-state Potts planar (XY) spins
C
C  The subroutines RAN2D(RADIUS,X1,Y1) and RAN3D(RADIUS,X1,Y1,Z1)
C  originate from the Program Library of CERN Computer Center
C  (CERN - Geneva), with original names RAN2VS and RAN3VS.
C
C  These subroutines call no subroutine.
C  They call the following function :
C  RANF() : this portable real function [Program library of CERN 
C           Computer Centre (CERN - Geneva)] is used to generate random 
C           numbers between 0 and 1. This function may be slower than
C           dedicated random number generators. Therefore, in order to 
C           save CPU time, users should replace RANF() by a call to
C           the equivalent function on the computer they use.
C           For instance, DEC-computer users should call the random 
C           generator RAN(I1,I2). For sake of convenience, the calls to
C           this function are included in the current version of the
C           program as comment cards. After implementing and testing
C           the program (see manual), the use of RAN(I1,I2) or of a
C           faster computer random generator, is recommended.
C
C***********************************************************************


      SUBROUTINE RAN1D(RADIUS,X1,Y1,Z1,N)

C***********************************************************************
C
C  For a general description of the subroutines RANnD(RADIUS,X1,Y1...),
C  see the comments just above.
C  RAN1D(RADIUS,X1) generates pseudo-randomly a number +RADIUS or
C  -RADIUS.
C
C***********************************************************************

      INCLUDE 'MCMAG.INC'

      DIMENSION
     &  D(NAMAX),DX(NAMAX),DY(NAMAX),DZ(NAMAX)

      COMMON
     &       /READ4/D,DX,DY,DZ

      INTEGER*2 I1,I2
      IF (RAN(I1,I2)-0.5.GT.0) THEN
C     IF (RANF()-0.5.GT.0) THEN
        X1 = RADIUS*DX(N)
        Y1 = RADIUS*DY(N)
        Z1 = RADIUS*DZ(N)
      ELSE
        X1 = -RADIUS*DX(N)
        Y1 = -RADIUS*DY(N)
        Z1 = -RADIUS*DZ(N)
      END IF

      RETURN

      END


      SUBROUTINE RAN2D(RADIUS,X1,Y1)

C***********************************************************************
C
C  For a general description of the subroutines RANnD(RADIUS,X1,Y1...),
C  see the comments just before the subroutine RAN1D(RADIUS,X1).
C  RAN2D(RADIUS,X1,Y1) generates pseudo-randomly a point on a circle of
C  radius RADIUS.
C  It originates from the Program Library of CERN Computer Center
C  (CERN - Geneva) with the original name RAN2VS.
C
C***********************************************************************


  10      X  =  2.*RAN(I1,I2) - 1.  
          Y  =  2.*RAN(I1,I2) - 1.  
C 10      X  =  2.*RANF() - 1.  
C         Y  =  2.*RANF() - 1.  
          RR =  X*X + Y*Y   
          IF(RR .GT. 1.)  GOTO 10   
          SCALE  =  RADIUS / SQRT(RR)   
          X1  =  X * SCALE  
          Y1  =  Y * SCALE  
          RETURN    
          END   


          SUBROUTINE RAN3D(RADIUS,X1,Y1,Z1)

C***********************************************************************
C
C  For a general description of the subroutines RANnD(RADIUS,X1,Y1...),
C  see the comments just before the subroutine RAN1D(RADIUS,X1).
C  RAN3D(RADIUS,X1,Y1,Z1) generates pseudo-randomly a point at the
C  surface of a sphere of radius RADIUS.
C  It originates from the Program Library of CERN Computer Center
C  (CERN - Geneva) with the original name RAN3VS.
C
C***********************************************************************


  10      X  =  2.*RAN(I1,I2) - 1.  
          Y  =  2.*RAN(I1,I2) - 1.  
          Z  =  2.*RAN(I1,I2) - 1.  
C 10      X  =  2.*RANF() - 1.  
C         Y  =  2.*RANF() - 1.  
C         Z  =  2.*RANF() - 1.  
          RR =  X*X + Y*Y + Z*Z 
          IF(RR .GT. 1.)  GOTO 10   
          SCALE  =  RADIUS / SQRT(RR)   
          X1  =  X * SCALE  
          Y1  =  Y * SCALE  
          Z1  =  Z * SCALE  
          RETURN    
          END   



      SUBROUTINE RANqD(RADIUS,X1,Y1)

C***********************************************************************
C
C  For a general description of the subroutines RANnD(RADIUS,X1,Y1...),
C  see the comments just before the subroutine RAN1D(RADIUS,X1).
C  RANqD(RADIUS,X1,Y1) generates pseudo-randomly a point on a circle of 
C  radius RADIUS among q points spaced out of an angle 360/q degrees.
C
C***********************************************************************


      INTEGER*2 I1,I2

      REAL*8 PI

      COMMON /RAND/PI,NQ

      URAN=RAN(I1,I2)
C     URAN=RANF()
      DO I=1,NQ
        AQ = (FLOAT(I))/(FLOAT(NQ))
        IF (URAN.LT.AQ) THEN
          X1 = RADIUS*COS(2*PI*(I-1)/NQ)
          Y1 = RADIUS*SIN(2*PI*(I-1)/NQ)
          GO TO 1
        END IF
      END DO

    1 RETURN

      END


      REAL FUNCTION RANF()

C***********************************************************************
C
C  This function generates a random number between 0 and 1.
C  It originates from the Program library of CERN Computer Centre 
C  (CERN - Geneva).
C  To optimize computing time, users should replace all calls to RANF()
C  by a call to the random number generator of the computer they use.
C
C***********************************************************************


          DOUBLE PRECISION    DRANF,    G900GT,   G900ST    
          DOUBLE PRECISION    DS(2),    DM(2),    DSEED 
          DOUBLE PRECISION    DX24,     DX48    
          DOUBLE PRECISION    DL,       DC,       DU,       DR  
          LOGICAL             SINGLE    
          DATA      DS     /  1665 1885.D0, 286 8876.D0  /  
          DATA      DM     /  1518 4245.D0, 265 1554.D0  /  
          DATA      DX24   /  1677 7216.D0  /   
          DATA      DX48   /  281 4749 7671 0656.D0  /  
          SINGLE  =  .TRUE. 
          GOTO 10   
          ENTRY DRANF() 
          SINGLE  =  .FALSE.    
  10      DL  =  DS(1) * DM(1)  
          DC  =  DINT(DL/DX24)  
          DL  =  DL - DC*DX24   
          DU  =  DS(1)*DM(2) + DS(2)*DM(1) + DC 
          DS(2)  =  DU - DINT(DU/DX24)*DX24 
          DS(1)  =  DL  
          DR     =  (DS(2)*DX24 + DS(1)) / DX48 
          IF(SINGLE)  THEN  
             RANF  =  SNGL(DR)  
          ELSE  
             DRANF  =  DR   
          ENDIF 
          RETURN    
          ENTRY G900GT()    
          G900GT  =  DS(2)*DX24 + DS(1) 
          RETURN    
          ENTRY G900ST(DSEED)   
          DS(2)  =  DINT(DSEED/DX24)    
          DS(1)  =  DSEED - DS(2)*DX24  
          G900ST =  DS(1)   
          RETURN    
          END   

